# -*- coding: utf-8 -*-
# **********************************************************************
#
# Copyright (c) 2003-2017 ZeroC, Inc. All rights reserved.
#
# This copy of Ice is licensed to you under the terms described in the
# ICE_LICENSE file included in this distribution.
#
# **********************************************************************
#
# Ice version 3.6.4
#
# <auto-generated>
#
# Generated from file `API.ice'
#
# Warning: do not edit this file.
#
# </auto-generated>
#

from sys import version_info as _version_info_
import Ice, IcePy
import omero_RTypes_ice
import omero_ServerErrors_ice

# Included module omero
_M_omero = Ice.openModule('omero')

# Included module Ice
_M_Ice = Ice.openModule('Ice')

# Included module Glacier2
_M_Glacier2 = Ice.openModule('Glacier2')

# Included module omero.model
_M_omero.model = Ice.openModule('omero.model')

# Included module omero.sys
_M_omero.sys = Ice.openModule('omero.sys')

# Included module omero.api
_M_omero.api = Ice.openModule('omero.api')

# Start of module omero
__name__ = 'omero'

# Start of module omero.cmd
_M_omero.cmd = Ice.openModule('omero.cmd')
__name__ = 'omero.cmd'
_M_omero.cmd.__doc__ = """
Simplified API that is intended for passing
"""

if '_t_StringMap' not in _M_omero.cmd.__dict__:
    _M_omero.cmd._t_StringMap = IcePy.defineDictionary('::omero::cmd::StringMap', (), IcePy._t_string, IcePy._t_string)

if '_t_StringMapList' not in _M_omero.cmd.__dict__:
    _M_omero.cmd._t_StringMapList = IcePy.defineSequence('::omero::cmd::StringMapList', (), _M_omero.cmd._t_StringMap)

if 'State' not in _M_omero.cmd.__dict__:
    _M_omero.cmd.State = Ice.createTempClass()
    class State(Ice.EnumBase):

        def __init__(self, _n, _v):
            Ice.EnumBase.__init__(self, _n, _v)

        def valueOf(self, _n):
            if _n in self._enumerators:
                return self._enumerators[_n]
            return None
        valueOf = classmethod(valueOf)

    State.ALL = State("ALL", 0)
    State.ACTIVE = State("ACTIVE", 1)
    State.INACTIVE = State("INACTIVE", 2)
    State.SUCCESS = State("SUCCESS", 3)
    State.FAILURE = State("FAILURE", 4)
    State.CANCELLED = State("CANCELLED", 5)
    State._enumerators = { 0:State.ALL, 1:State.ACTIVE, 2:State.INACTIVE, 3:State.SUCCESS, 4:State.FAILURE, 5:State.CANCELLED }

    _M_omero.cmd._t_State = IcePy.defineEnum('::omero::cmd::State', State, (), State._enumerators)

    _M_omero.cmd.State = State
    del State

if '_t_StateList' not in _M_omero.cmd.__dict__:
    _M_omero.cmd._t_StateList = IcePy.defineSequence('::omero::cmd::StateList', (), _M_omero.cmd._t_State)

if 'Handle' not in _M_omero.cmd.__dict__:
    _M_omero.cmd._t_Handle = IcePy.declareClass('::omero::cmd::Handle')
    _M_omero.cmd._t_HandlePrx = IcePy.declareProxy('::omero::cmd::Handle')

if 'Status' not in _M_omero.cmd.__dict__:
    _M_omero.cmd.Status = Ice.createTempClass()
    class Status(Ice.Object):
        def __init__(self, source=None, category='', name='', flags=None, parameters=None, currentStep=0, steps=0, startTime=0, stepStartTimes=None, stepStopTimes=None, stopTime=0):
            self.source = source
            self.category = category
            self.name = name
            self.flags = flags
            self.parameters = parameters
            self.currentStep = currentStep
            self.steps = steps
            self.startTime = startTime
            self.stepStartTimes = stepStartTimes
            self.stepStopTimes = stepStopTimes
            self.stopTime = stopTime

        def ice_ids(self, current=None):
            return ('::Ice::Object', '::omero::cmd::Status')

        def ice_id(self, current=None):
            return '::omero::cmd::Status'

        def ice_staticId():
            return '::omero::cmd::Status'
        ice_staticId = staticmethod(ice_staticId)

        def __str__(self):
            return IcePy.stringify(self, _M_omero.cmd._t_Status)

        __repr__ = __str__

    _M_omero.cmd.StatusPrx = Ice.createTempClass()
    class StatusPrx(Ice.ObjectPrx):

        def checkedCast(proxy, facetOrCtx=None, _ctx=None):
            return _M_omero.cmd.StatusPrx.ice_checkedCast(proxy, '::omero::cmd::Status', facetOrCtx, _ctx)
        checkedCast = staticmethod(checkedCast)

        def uncheckedCast(proxy, facet=None):
            return _M_omero.cmd.StatusPrx.ice_uncheckedCast(proxy, facet)
        uncheckedCast = staticmethod(uncheckedCast)

        def ice_staticId():
            return '::omero::cmd::Status'
        ice_staticId = staticmethod(ice_staticId)

    _M_omero.cmd._t_StatusPrx = IcePy.defineProxy('::omero::cmd::Status', StatusPrx)

    _M_omero.cmd._t_Status = IcePy.defineClass('::omero::cmd::Status', Status, -1, (), False, False, None, (), (
        ('source', (), _M_omero.cmd._t_HandlePrx, False, 0),
        ('category', (), IcePy._t_string, False, 0),
        ('name', (), IcePy._t_string, False, 0),
        ('flags', (), _M_omero.cmd._t_StateList, False, 0),
        ('parameters', (), _M_omero.cmd._t_StringMap, False, 0),
        ('currentStep', (), IcePy._t_int, False, 0),
        ('steps', (), IcePy._t_int, False, 0),
        ('startTime', (), IcePy._t_long, False, 0),
        ('stepStartTimes', (), _M_Ice._t_LongSeq, False, 0),
        ('stepStopTimes', (), _M_Ice._t_LongSeq, False, 0),
        ('stopTime', (), IcePy._t_long, False, 0)
    ))
    Status._ice_type = _M_omero.cmd._t_Status

    _M_omero.cmd.Status = Status
    del Status

    _M_omero.cmd.StatusPrx = StatusPrx
    del StatusPrx

if '_t_StatusList' not in _M_omero.cmd.__dict__:
    _M_omero.cmd._t_StatusList = IcePy.defineSequence('::omero::cmd::StatusList', (), _M_omero.cmd._t_Status)

if 'Request' not in _M_omero.cmd.__dict__:
    _M_omero.cmd.Request = Ice.createTempClass()
    class Request(Ice.Object):
        def __init__(self):
            pass

        def ice_ids(self, current=None):
            return ('::Ice::Object', '::omero::cmd::Request')

        def ice_id(self, current=None):
            return '::omero::cmd::Request'

        def ice_staticId():
            return '::omero::cmd::Request'
        ice_staticId = staticmethod(ice_staticId)

        def __str__(self):
            return IcePy.stringify(self, _M_omero.cmd._t_Request)

        __repr__ = __str__

    _M_omero.cmd.RequestPrx = Ice.createTempClass()
    class RequestPrx(Ice.ObjectPrx):

        def checkedCast(proxy, facetOrCtx=None, _ctx=None):
            return _M_omero.cmd.RequestPrx.ice_checkedCast(proxy, '::omero::cmd::Request', facetOrCtx, _ctx)
        checkedCast = staticmethod(checkedCast)

        def uncheckedCast(proxy, facet=None):
            return _M_omero.cmd.RequestPrx.ice_uncheckedCast(proxy, facet)
        uncheckedCast = staticmethod(uncheckedCast)

        def ice_staticId():
            return '::omero::cmd::Request'
        ice_staticId = staticmethod(ice_staticId)

    _M_omero.cmd._t_RequestPrx = IcePy.defineProxy('::omero::cmd::Request', RequestPrx)

    _M_omero.cmd._t_Request = IcePy.defineClass('::omero::cmd::Request', Request, -1, (), False, False, None, (), ())
    Request._ice_type = _M_omero.cmd._t_Request

    _M_omero.cmd.Request = Request
    del Request

    _M_omero.cmd.RequestPrx = RequestPrx
    del RequestPrx

if '_t_RequestList' not in _M_omero.cmd.__dict__:
    _M_omero.cmd._t_RequestList = IcePy.defineSequence('::omero::cmd::RequestList', (), _M_omero.cmd._t_Request)

if 'Response' not in _M_omero.cmd.__dict__:
    _M_omero.cmd.Response = Ice.createTempClass()
    class Response(Ice.Object):
        def __init__(self):
            pass

        def ice_ids(self, current=None):
            return ('::Ice::Object', '::omero::cmd::Response')

        def ice_id(self, current=None):
            return '::omero::cmd::Response'

        def ice_staticId():
            return '::omero::cmd::Response'
        ice_staticId = staticmethod(ice_staticId)

        def __str__(self):
            return IcePy.stringify(self, _M_omero.cmd._t_Response)

        __repr__ = __str__

    _M_omero.cmd.ResponsePrx = Ice.createTempClass()
    class ResponsePrx(Ice.ObjectPrx):

        def checkedCast(proxy, facetOrCtx=None, _ctx=None):
            return _M_omero.cmd.ResponsePrx.ice_checkedCast(proxy, '::omero::cmd::Response', facetOrCtx, _ctx)
        checkedCast = staticmethod(checkedCast)

        def uncheckedCast(proxy, facet=None):
            return _M_omero.cmd.ResponsePrx.ice_uncheckedCast(proxy, facet)
        uncheckedCast = staticmethod(uncheckedCast)

        def ice_staticId():
            return '::omero::cmd::Response'
        ice_staticId = staticmethod(ice_staticId)

    _M_omero.cmd._t_ResponsePrx = IcePy.defineProxy('::omero::cmd::Response', ResponsePrx)

    _M_omero.cmd._t_Response = IcePy.defineClass('::omero::cmd::Response', Response, -1, (), False, False, None, (), ())
    Response._ice_type = _M_omero.cmd._t_Response

    _M_omero.cmd.Response = Response
    del Response

    _M_omero.cmd.ResponsePrx = ResponsePrx
    del ResponsePrx

if 'OK' not in _M_omero.cmd.__dict__:
    _M_omero.cmd.OK = Ice.createTempClass()
    class OK(_M_omero.cmd.Response):
        def __init__(self):
            _M_omero.cmd.Response.__init__(self)

        def ice_ids(self, current=None):
            return ('::Ice::Object', '::omero::cmd::OK', '::omero::cmd::Response')

        def ice_id(self, current=None):
            return '::omero::cmd::OK'

        def ice_staticId():
            return '::omero::cmd::OK'
        ice_staticId = staticmethod(ice_staticId)

        def __str__(self):
            return IcePy.stringify(self, _M_omero.cmd._t_OK)

        __repr__ = __str__

    _M_omero.cmd.OKPrx = Ice.createTempClass()
    class OKPrx(_M_omero.cmd.ResponsePrx):

        def checkedCast(proxy, facetOrCtx=None, _ctx=None):
            return _M_omero.cmd.OKPrx.ice_checkedCast(proxy, '::omero::cmd::OK', facetOrCtx, _ctx)
        checkedCast = staticmethod(checkedCast)

        def uncheckedCast(proxy, facet=None):
            return _M_omero.cmd.OKPrx.ice_uncheckedCast(proxy, facet)
        uncheckedCast = staticmethod(uncheckedCast)

        def ice_staticId():
            return '::omero::cmd::OK'
        ice_staticId = staticmethod(ice_staticId)

    _M_omero.cmd._t_OKPrx = IcePy.defineProxy('::omero::cmd::OK', OKPrx)

    _M_omero.cmd._t_OK = IcePy.defineClass('::omero::cmd::OK', OK, -1, (), False, False, _M_omero.cmd._t_Response, (), ())
    OK._ice_type = _M_omero.cmd._t_OK

    _M_omero.cmd.OK = OK
    del OK

    _M_omero.cmd.OKPrx = OKPrx
    del OKPrx

if 'ERR' not in _M_omero.cmd.__dict__:
    _M_omero.cmd.ERR = Ice.createTempClass()
    class ERR(_M_omero.cmd.Response):
        def __init__(self, category='', name='', parameters=None):
            _M_omero.cmd.Response.__init__(self)
            self.category = category
            self.name = name
            self.parameters = parameters

        def ice_ids(self, current=None):
            return ('::Ice::Object', '::omero::cmd::ERR', '::omero::cmd::Response')

        def ice_id(self, current=None):
            return '::omero::cmd::ERR'

        def ice_staticId():
            return '::omero::cmd::ERR'
        ice_staticId = staticmethod(ice_staticId)

        def __str__(self):
            return IcePy.stringify(self, _M_omero.cmd._t_ERR)

        __repr__ = __str__

    _M_omero.cmd.ERRPrx = Ice.createTempClass()
    class ERRPrx(_M_omero.cmd.ResponsePrx):

        def checkedCast(proxy, facetOrCtx=None, _ctx=None):
            return _M_omero.cmd.ERRPrx.ice_checkedCast(proxy, '::omero::cmd::ERR', facetOrCtx, _ctx)
        checkedCast = staticmethod(checkedCast)

        def uncheckedCast(proxy, facet=None):
            return _M_omero.cmd.ERRPrx.ice_uncheckedCast(proxy, facet)
        uncheckedCast = staticmethod(uncheckedCast)

        def ice_staticId():
            return '::omero::cmd::ERR'
        ice_staticId = staticmethod(ice_staticId)

    _M_omero.cmd._t_ERRPrx = IcePy.defineProxy('::omero::cmd::ERR', ERRPrx)

    _M_omero.cmd._t_ERR = IcePy.defineClass('::omero::cmd::ERR', ERR, -1, (), False, False, _M_omero.cmd._t_Response, (), (
        ('category', (), IcePy._t_string, False, 0),
        ('name', (), IcePy._t_string, False, 0),
        ('parameters', (), _M_omero.cmd._t_StringMap, False, 0)
    ))
    ERR._ice_type = _M_omero.cmd._t_ERR

    _M_omero.cmd.ERR = ERR
    del ERR

    _M_omero.cmd.ERRPrx = ERRPrx
    del ERRPrx

if 'Unknown' not in _M_omero.cmd.__dict__:
    _M_omero.cmd.Unknown = Ice.createTempClass()
    class Unknown(_M_omero.cmd.ERR):
        def __init__(self, category='', name='', parameters=None):
            _M_omero.cmd.ERR.__init__(self, category, name, parameters)

        def ice_ids(self, current=None):
            return ('::Ice::Object', '::omero::cmd::ERR', '::omero::cmd::Response', '::omero::cmd::Unknown')

        def ice_id(self, current=None):
            return '::omero::cmd::Unknown'

        def ice_staticId():
            return '::omero::cmd::Unknown'
        ice_staticId = staticmethod(ice_staticId)

        def __str__(self):
            return IcePy.stringify(self, _M_omero.cmd._t_Unknown)

        __repr__ = __str__

    _M_omero.cmd.UnknownPrx = Ice.createTempClass()
    class UnknownPrx(_M_omero.cmd.ERRPrx):

        def checkedCast(proxy, facetOrCtx=None, _ctx=None):
            return _M_omero.cmd.UnknownPrx.ice_checkedCast(proxy, '::omero::cmd::Unknown', facetOrCtx, _ctx)
        checkedCast = staticmethod(checkedCast)

        def uncheckedCast(proxy, facet=None):
            return _M_omero.cmd.UnknownPrx.ice_uncheckedCast(proxy, facet)
        uncheckedCast = staticmethod(uncheckedCast)

        def ice_staticId():
            return '::omero::cmd::Unknown'
        ice_staticId = staticmethod(ice_staticId)

    _M_omero.cmd._t_UnknownPrx = IcePy.defineProxy('::omero::cmd::Unknown', UnknownPrx)

    _M_omero.cmd._t_Unknown = IcePy.defineClass('::omero::cmd::Unknown', Unknown, -1, (), False, False, _M_omero.cmd._t_ERR, (), ())
    Unknown._ice_type = _M_omero.cmd._t_Unknown

    _M_omero.cmd.Unknown = Unknown
    del Unknown

    _M_omero.cmd.UnknownPrx = UnknownPrx
    del UnknownPrx

if '_t_ResponseList' not in _M_omero.cmd.__dict__:
    _M_omero.cmd._t_ResponseList = IcePy.defineSequence('::omero::cmd::ResponseList', (), _M_omero.cmd._t_Response)

if 'CmdCallback' not in _M_omero.cmd.__dict__:
    _M_omero.cmd.CmdCallback = Ice.createTempClass()
    class CmdCallback(Ice.Object):
        def __init__(self):
            if Ice.getType(self) == _M_omero.cmd.CmdCallback:
                raise RuntimeError('omero.cmd.CmdCallback is an abstract class')

        def ice_ids(self, current=None):
            return ('::Ice::Object', '::omero::cmd::CmdCallback')

        def ice_id(self, current=None):
            return '::omero::cmd::CmdCallback'

        def ice_staticId():
            return '::omero::cmd::CmdCallback'
        ice_staticId = staticmethod(ice_staticId)

        def step(self, complete, total, current=None):
            """
            Notifies clients that the given number of steps
            from the total is complete. This method will not
            necessarily be called for every step.
            Arguments:
            complete -- 
            total -- 
            current -- The Current object for the invocation.
            """
            pass

        def finished(self, rsp, s, current=None):
            """
            Called when the command has completed in any fashion
            including cancellation. The Status#flags list will
            contain information about whether or not the process
            was cancelled.
            Arguments:
            rsp -- 
            s -- 
            current -- The Current object for the invocation.
            """
            pass

        def __str__(self):
            return IcePy.stringify(self, _M_omero.cmd._t_CmdCallback)

        __repr__ = __str__

    _M_omero.cmd.CmdCallbackPrx = Ice.createTempClass()
    class CmdCallbackPrx(Ice.ObjectPrx):

        """
        Notifies clients that the given number of steps
        from the total is complete. This method will not
        necessarily be called for every step.
        Arguments:
        complete -- 
        total -- 
        _ctx -- The request context for the invocation.
        """
        def step(self, complete, total, _ctx=None):
            return _M_omero.cmd.CmdCallback._op_step.invoke(self, ((complete, total), _ctx))

        """
        Notifies clients that the given number of steps
        from the total is complete. This method will not
        necessarily be called for every step.
        Arguments:
        complete -- 
        total -- 
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_step(self, complete, total, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.cmd.CmdCallback._op_step.begin(self, ((complete, total), _response, _ex, _sent, _ctx))

        """
        Notifies clients that the given number of steps
        from the total is complete. This method will not
        necessarily be called for every step.
        Arguments:
        complete -- 
        total -- 
        """
        def end_step(self, _r):
            return _M_omero.cmd.CmdCallback._op_step.end(self, _r)

        """
        Called when the command has completed in any fashion
        including cancellation. The Status#flags list will
        contain information about whether or not the process
        was cancelled.
        Arguments:
        rsp -- 
        s -- 
        _ctx -- The request context for the invocation.
        """
        def finished(self, rsp, s, _ctx=None):
            return _M_omero.cmd.CmdCallback._op_finished.invoke(self, ((rsp, s), _ctx))

        """
        Called when the command has completed in any fashion
        including cancellation. The Status#flags list will
        contain information about whether or not the process
        was cancelled.
        Arguments:
        rsp -- 
        s -- 
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_finished(self, rsp, s, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.cmd.CmdCallback._op_finished.begin(self, ((rsp, s), _response, _ex, _sent, _ctx))

        """
        Called when the command has completed in any fashion
        including cancellation. The Status#flags list will
        contain information about whether or not the process
        was cancelled.
        Arguments:
        rsp -- 
        s -- 
        """
        def end_finished(self, _r):
            return _M_omero.cmd.CmdCallback._op_finished.end(self, _r)

        def checkedCast(proxy, facetOrCtx=None, _ctx=None):
            return _M_omero.cmd.CmdCallbackPrx.ice_checkedCast(proxy, '::omero::cmd::CmdCallback', facetOrCtx, _ctx)
        checkedCast = staticmethod(checkedCast)

        def uncheckedCast(proxy, facet=None):
            return _M_omero.cmd.CmdCallbackPrx.ice_uncheckedCast(proxy, facet)
        uncheckedCast = staticmethod(uncheckedCast)

        def ice_staticId():
            return '::omero::cmd::CmdCallback'
        ice_staticId = staticmethod(ice_staticId)

    _M_omero.cmd._t_CmdCallbackPrx = IcePy.defineProxy('::omero::cmd::CmdCallback', CmdCallbackPrx)

    _M_omero.cmd._t_CmdCallback = IcePy.defineClass('::omero::cmd::CmdCallback', CmdCallback, -1, (), True, False, None, (), ())
    CmdCallback._ice_type = _M_omero.cmd._t_CmdCallback

    CmdCallback._op_step = IcePy.Operation('step', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), IcePy._t_int, False, 0), ((), IcePy._t_int, False, 0)), (), None, ())
    CmdCallback._op_finished = IcePy.Operation('finished', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.cmd._t_Response, False, 0), ((), _M_omero.cmd._t_Status, False, 0)), (), None, ())

    _M_omero.cmd.CmdCallback = CmdCallback
    del CmdCallback

    _M_omero.cmd.CmdCallbackPrx = CmdCallbackPrx
    del CmdCallbackPrx

if 'Handle' not in _M_omero.cmd.__dict__:
    _M_omero.cmd.Handle = Ice.createTempClass()
    class Handle(Ice.Object):
        def __init__(self):
            if Ice.getType(self) == _M_omero.cmd.Handle:
                raise RuntimeError('omero.cmd.Handle is an abstract class')

        def ice_ids(self, current=None):
            return ('::Ice::Object', '::omero::cmd::Handle')

        def ice_id(self, current=None):
            return '::omero::cmd::Handle'

        def ice_staticId():
            return '::omero::cmd::Handle'
        ice_staticId = staticmethod(ice_staticId)

        def addCallback(self, cb, current=None):
            """
            Add a callback for notifications.
            Arguments:
            cb -- 
            current -- The Current object for the invocation.
            """
            pass

        def removeCallback(self, cb, current=None):
            """
            Remove callback for notifications.
            Arguments:
            cb -- 
            current -- The Current object for the invocation.
            """
            pass

        def getRequest(self, current=None):
            """
            Returns the request object that was used to
            initialize this handle. Never null.
            Arguments:
            current -- The Current object for the invocation.
            """
            pass

        def getResponse(self, current=None):
            """
            Returns a response if this handle has finished
            execution, otherwise returns null.
            Arguments:
            current -- The Current object for the invocation.
            """
            pass

        def getStatus(self, current=None):
            """
            Returns a status object for the current execution.
            This will likely be the same object that would be
            returned as a component of the Response value.
            Never null.
            Arguments:
            current -- The Current object for the invocation.
            """
            pass

        def cancel(self, current=None):
            """
            Attempts to cancel execution of this Request. Returns
            true if cancellation was successful. Returns false if not,
            in which case likely this request will run to completion.
            Arguments:
            current -- The Current object for the invocation.
            """
            pass

        def close(self, current=None):
            """
            Closes this handle. If the request is running, then a
            cancellation will be attempted first. All uses of a handle
            should be surrounded by a try/finally close block.
            Arguments:
            current -- The Current object for the invocation.
            """
            pass

        def __str__(self):
            return IcePy.stringify(self, _M_omero.cmd._t_Handle)

        __repr__ = __str__

    _M_omero.cmd.HandlePrx = Ice.createTempClass()
    class HandlePrx(Ice.ObjectPrx):

        """
        Add a callback for notifications.
        Arguments:
        cb -- 
        _ctx -- The request context for the invocation.
        """
        def addCallback(self, cb, _ctx=None):
            return _M_omero.cmd.Handle._op_addCallback.invoke(self, ((cb, ), _ctx))

        """
        Add a callback for notifications.
        Arguments:
        cb -- 
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_addCallback(self, cb, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.cmd.Handle._op_addCallback.begin(self, ((cb, ), _response, _ex, _sent, _ctx))

        """
        Add a callback for notifications.
        Arguments:
        cb -- 
        """
        def end_addCallback(self, _r):
            return _M_omero.cmd.Handle._op_addCallback.end(self, _r)

        """
        Remove callback for notifications.
        Arguments:
        cb -- 
        _ctx -- The request context for the invocation.
        """
        def removeCallback(self, cb, _ctx=None):
            return _M_omero.cmd.Handle._op_removeCallback.invoke(self, ((cb, ), _ctx))

        """
        Remove callback for notifications.
        Arguments:
        cb -- 
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_removeCallback(self, cb, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.cmd.Handle._op_removeCallback.begin(self, ((cb, ), _response, _ex, _sent, _ctx))

        """
        Remove callback for notifications.
        Arguments:
        cb -- 
        """
        def end_removeCallback(self, _r):
            return _M_omero.cmd.Handle._op_removeCallback.end(self, _r)

        """
        Returns the request object that was used to
        initialize this handle. Never null.
        Arguments:
        _ctx -- The request context for the invocation.
        """
        def getRequest(self, _ctx=None):
            return _M_omero.cmd.Handle._op_getRequest.invoke(self, ((), _ctx))

        """
        Returns the request object that was used to
        initialize this handle. Never null.
        Arguments:
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_getRequest(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.cmd.Handle._op_getRequest.begin(self, ((), _response, _ex, _sent, _ctx))

        """
        Returns the request object that was used to
        initialize this handle. Never null.
        Arguments:
        """
        def end_getRequest(self, _r):
            return _M_omero.cmd.Handle._op_getRequest.end(self, _r)

        """
        Returns a response if this handle has finished
        execution, otherwise returns null.
        Arguments:
        _ctx -- The request context for the invocation.
        """
        def getResponse(self, _ctx=None):
            return _M_omero.cmd.Handle._op_getResponse.invoke(self, ((), _ctx))

        """
        Returns a response if this handle has finished
        execution, otherwise returns null.
        Arguments:
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_getResponse(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.cmd.Handle._op_getResponse.begin(self, ((), _response, _ex, _sent, _ctx))

        """
        Returns a response if this handle has finished
        execution, otherwise returns null.
        Arguments:
        """
        def end_getResponse(self, _r):
            return _M_omero.cmd.Handle._op_getResponse.end(self, _r)

        """
        Returns a status object for the current execution.
        This will likely be the same object that would be
        returned as a component of the Response value.
        Never null.
        Arguments:
        _ctx -- The request context for the invocation.
        """
        def getStatus(self, _ctx=None):
            return _M_omero.cmd.Handle._op_getStatus.invoke(self, ((), _ctx))

        """
        Returns a status object for the current execution.
        This will likely be the same object that would be
        returned as a component of the Response value.
        Never null.
        Arguments:
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_getStatus(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.cmd.Handle._op_getStatus.begin(self, ((), _response, _ex, _sent, _ctx))

        """
        Returns a status object for the current execution.
        This will likely be the same object that would be
        returned as a component of the Response value.
        Never null.
        Arguments:
        """
        def end_getStatus(self, _r):
            return _M_omero.cmd.Handle._op_getStatus.end(self, _r)

        """
        Attempts to cancel execution of this Request. Returns
        true if cancellation was successful. Returns false if not,
        in which case likely this request will run to completion.
        Arguments:
        _ctx -- The request context for the invocation.
        """
        def cancel(self, _ctx=None):
            return _M_omero.cmd.Handle._op_cancel.invoke(self, ((), _ctx))

        """
        Attempts to cancel execution of this Request. Returns
        true if cancellation was successful. Returns false if not,
        in which case likely this request will run to completion.
        Arguments:
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_cancel(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.cmd.Handle._op_cancel.begin(self, ((), _response, _ex, _sent, _ctx))

        """
        Attempts to cancel execution of this Request. Returns
        true if cancellation was successful. Returns false if not,
        in which case likely this request will run to completion.
        Arguments:
        """
        def end_cancel(self, _r):
            return _M_omero.cmd.Handle._op_cancel.end(self, _r)

        """
        Closes this handle. If the request is running, then a
        cancellation will be attempted first. All uses of a handle
        should be surrounded by a try/finally close block.
        Arguments:
        _ctx -- The request context for the invocation.
        """
        def close(self, _ctx=None):
            return _M_omero.cmd.Handle._op_close.invoke(self, ((), _ctx))

        """
        Closes this handle. If the request is running, then a
        cancellation will be attempted first. All uses of a handle
        should be surrounded by a try/finally close block.
        Arguments:
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_close(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.cmd.Handle._op_close.begin(self, ((), _response, _ex, _sent, _ctx))

        """
        Closes this handle. If the request is running, then a
        cancellation will be attempted first. All uses of a handle
        should be surrounded by a try/finally close block.
        Arguments:
        """
        def end_close(self, _r):
            return _M_omero.cmd.Handle._op_close.end(self, _r)

        def checkedCast(proxy, facetOrCtx=None, _ctx=None):
            return _M_omero.cmd.HandlePrx.ice_checkedCast(proxy, '::omero::cmd::Handle', facetOrCtx, _ctx)
        checkedCast = staticmethod(checkedCast)

        def uncheckedCast(proxy, facet=None):
            return _M_omero.cmd.HandlePrx.ice_uncheckedCast(proxy, facet)
        uncheckedCast = staticmethod(uncheckedCast)

        def ice_staticId():
            return '::omero::cmd::Handle'
        ice_staticId = staticmethod(ice_staticId)

    _M_omero.cmd._t_HandlePrx = IcePy.defineProxy('::omero::cmd::Handle', HandlePrx)

    _M_omero.cmd._t_Handle = IcePy.defineClass('::omero::cmd::Handle', Handle, -1, (), True, False, None, (), ())
    Handle._ice_type = _M_omero.cmd._t_Handle

    Handle._op_addCallback = IcePy.Operation('addCallback', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.cmd._t_CmdCallbackPrx, False, 0),), (), None, ())
    Handle._op_removeCallback = IcePy.Operation('removeCallback', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.cmd._t_CmdCallbackPrx, False, 0),), (), None, ())
    Handle._op_getRequest = IcePy.Operation('getRequest', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.cmd._t_Request, False, 0), ())
    Handle._op_getResponse = IcePy.Operation('getResponse', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.cmd._t_Response, False, 0), ())
    Handle._op_getStatus = IcePy.Operation('getStatus', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.cmd._t_Status, False, 0), ())
    Handle._op_cancel = IcePy.Operation('cancel', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), IcePy._t_bool, False, 0), (_M_omero._t_LockTimeout,))
    Handle._op_close = IcePy.Operation('close', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), None, ())

    _M_omero.cmd.Handle = Handle
    del Handle

    _M_omero.cmd.HandlePrx = HandlePrx
    del HandlePrx

if '_t_HandleList' not in _M_omero.cmd.__dict__:
    _M_omero.cmd._t_HandleList = IcePy.defineSequence('::omero::cmd::HandleList', (), _M_omero.cmd._t_HandlePrx)

if 'Session' not in _M_omero.cmd.__dict__:
    _M_omero.cmd.Session = Ice.createTempClass()
    class Session(_M_Glacier2.Session):
        """
        Starting point for all command-based OMERO.blitz interaction.
        """
        def __init__(self):
            if Ice.getType(self) == _M_omero.cmd.Session:
                raise RuntimeError('omero.cmd.Session is an abstract class')

        def ice_ids(self, current=None):
            return ('::Glacier2::Session', '::Ice::Object', '::omero::cmd::Session')

        def ice_id(self, current=None):
            return '::omero::cmd::Session'

        def ice_staticId():
            return '::omero::cmd::Session'
        ice_staticId = staticmethod(ice_staticId)

        def submit_async(self, _cb, req, current=None):
            pass

        def __str__(self):
            return IcePy.stringify(self, _M_omero.cmd._t_Session)

        __repr__ = __str__

    _M_omero.cmd.SessionPrx = Ice.createTempClass()
    class SessionPrx(_M_Glacier2.SessionPrx):

        def submit(self, req, _ctx=None):
            return _M_omero.cmd.Session._op_submit.invoke(self, ((req, ), _ctx))

        def begin_submit(self, req, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.cmd.Session._op_submit.begin(self, ((req, ), _response, _ex, _sent, _ctx))

        def end_submit(self, _r):
            return _M_omero.cmd.Session._op_submit.end(self, _r)

        def checkedCast(proxy, facetOrCtx=None, _ctx=None):
            return _M_omero.cmd.SessionPrx.ice_checkedCast(proxy, '::omero::cmd::Session', facetOrCtx, _ctx)
        checkedCast = staticmethod(checkedCast)

        def uncheckedCast(proxy, facet=None):
            return _M_omero.cmd.SessionPrx.ice_uncheckedCast(proxy, facet)
        uncheckedCast = staticmethod(uncheckedCast)

        def ice_staticId():
            return '::omero::cmd::Session'
        ice_staticId = staticmethod(ice_staticId)

    _M_omero.cmd._t_SessionPrx = IcePy.defineProxy('::omero::cmd::Session', SessionPrx)

    _M_omero.cmd._t_Session = IcePy.defineClass('::omero::cmd::Session', Session, -1, (), True, False, None, (_M_Glacier2._t_Session,), ())
    Session._ice_type = _M_omero.cmd._t_Session

    Session._op_submit = IcePy.Operation('submit', Ice.OperationMode.Normal, Ice.OperationMode.Normal, True, None, (), (((), _M_omero.cmd._t_Request, False, 0),), (), ((), _M_omero.cmd._t_HandlePrx, False, 0), ())

    _M_omero.cmd.Session = Session
    del Session

    _M_omero.cmd.SessionPrx = SessionPrx
    del SessionPrx

# End of module omero.cmd

__name__ = 'omero'

# End of module omero



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# http://svn.smedbergs.us/python-processes/trunk/winprocess.py
#

# A module to expose various thread/process/job related structures and
# methods from kernel32
#
# The MIT License
#
# Copyright (c) 2006 the Mozilla Foundation <http://www.mozilla.org>
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

from builtins import object
from ctypes import c_void_p, POINTER, sizeof, Structure, windll, WinError, WINFUNCTYPE
from ctypes.wintypes import BOOL, BYTE, DWORD, HANDLE, LPCWSTR, LPWSTR, UINT, WORD

LPVOID = c_void_p
LPBYTE = POINTER(BYTE)
LPDWORD = POINTER(DWORD)

def ErrCheckBool(result, func, args):
    """errcheck function for Windows functions that return a BOOL True
    on success"""
    if not result:
        raise WinError()
    return args

# CloseHandle()

CloseHandleProto = WINFUNCTYPE(BOOL, HANDLE)
CloseHandle = CloseHandleProto(("CloseHandle", windll.kernel32))
CloseHandle.errcheck = ErrCheckBool

# AutoHANDLE

class AutoHANDLE(HANDLE):
    """Subclass of HANDLE which will call CloseHandle() on deletion."""
    def Close(self):
        if self.value:
            CloseHandle(self)
            self.value = 0
    
    def __del__(self):
        self.Close()

    def __int__(self):
        return self.value

def ErrCheckHandle(result, func, args):
    """errcheck function for Windows functions that return a HANDLE."""
    if not result:
        raise WinError()
    return AutoHANDLE(result)

# PROCESS_INFORMATION structure

class PROCESS_INFORMATION(Structure):
    _fields_ = [("hProcess", HANDLE),
                ("hThread", HANDLE),
                ("dwProcessID", DWORD),
                ("dwThreadID", DWORD)]

    def __init__(self):
        Structure.__init__(self)
        
        self.cb = sizeof(self)

LPPROCESS_INFORMATION = POINTER(PROCESS_INFORMATION)

# STARTUPINFO structure

class STARTUPINFO(Structure):
    _fields_ = [("cb", DWORD),
                ("lpReserved", LPWSTR),
                ("lpDesktop", LPWSTR),
                ("lpTitle", LPWSTR),
                ("dwX", DWORD),
                ("dwY", DWORD),
                ("dwXSize", DWORD),
                ("dwYSize", DWORD),
                ("dwXCountChars", DWORD),
                ("dwYCountChars", DWORD),
                ("dwFillAttribute", DWORD),
                ("dwFlags", DWORD),
                ("wShowWindow", WORD),
                ("cbReserved2", WORD),
                ("lpReserved2", LPBYTE),
                ("hStdInput", HANDLE),
                ("hStdOutput", HANDLE),
                ("hStdError", HANDLE)
                ]
LPSTARTUPINFO = POINTER(STARTUPINFO)

STARTF_USESHOWWINDOW    = 0x01
STARTF_USESIZE          = 0x02
STARTF_USEPOSITION      = 0x04
STARTF_USECOUNTCHARS    = 0x08
STARTF_USEFILLATTRIBUTE = 0x10
STARTF_RUNFULLSCREEN    = 0x20
STARTF_FORCEONFEEDBACK  = 0x40
STARTF_FORCEOFFFEEDBACK = 0x80
STARTF_USESTDHANDLES    = 0x100

# EnvironmentBlock

class EnvironmentBlock(object):
    """An object which can be passed as the lpEnv parameter of CreateProcess.
    It is initialized with a dictionary."""

    def __init__(self, dict):
        if not dict:
            self._as_parameter_ = None
        else:
            values = ["%s=%s" % (key, value)
                      for (key, value) in dict.items()]
            values.append("")
            self._as_parameter_ = LPCWSTR("\0".join(values))
        
# CreateProcess()

CreateProcessProto = WINFUNCTYPE(BOOL,                  # Return type
                                 LPCWSTR,               # lpApplicationName
                                 LPWSTR,                # lpCommandLine
                                 LPVOID,                # lpProcessAttributes
                                 LPVOID,                # lpThreadAttributes
                                 BOOL,                  # bInheritHandles
                                 DWORD,                 # dwCreationFlags
                                 LPVOID,                # lpEnvironment
                                 LPCWSTR,               # lpCurrentDirectory
                                 LPSTARTUPINFO,         # lpStartupInfo
                                 LPPROCESS_INFORMATION  # lpProcessInformation
                                 )

CreateProcessFlags = ((1, "lpApplicationName", None),
                      (1, "lpCommandLine"),
                      (1, "lpProcessAttributes", None),
                      (1, "lpThreadAttributes", None),
                      (1, "bInheritHandles", True),
                      (1, "dwCreationFlags", 0),
                      (1, "lpEnvironment", None),
                      (1, "lpCurrentDirectory", None),
                      (1, "lpStartupInfo"),
                      (2, "lpProcessInformation"))

def ErrCheckCreateProcess(result, func, args):
    ErrCheckBool(result, func, args)
    # return a tuple (hProcess, hThread, dwProcessID, dwThreadID)
    pi = args[9]
    return AutoHANDLE(pi.hProcess), AutoHANDLE(pi.hThread), pi.dwProcessID, pi.dwThreadID

CreateProcess = CreateProcessProto(("CreateProcessW", windll.kernel32),
                                   CreateProcessFlags)
CreateProcess.errcheck = ErrCheckCreateProcess

CREATE_BREAKAWAY_FROM_JOB = 0x01000000
CREATE_DEFAULT_ERROR_MODE = 0x04000000
CREATE_NEW_CONSOLE = 0x00000010
CREATE_NEW_PROCESS_GROUP = 0x00000200
CREATE_NO_WINDOW = 0x08000000
CREATE_SUSPENDED = 0x00000004
CREATE_UNICODE_ENVIRONMENT = 0x00000400
DEBUG_ONLY_THIS_PROCESS = 0x00000002
DEBUG_PROCESS = 0x00000001
DETACHED_PROCESS = 0x00000008

# CreateJobObject()

CreateJobObjectProto = WINFUNCTYPE(HANDLE,             # Return type
                                   LPVOID,             # lpJobAttributes
                                   LPCWSTR             # lpName
                                   )

CreateJobObjectFlags = ((1, "lpJobAttributes", None),
                        (1, "lpName", None))

CreateJobObject = CreateJobObjectProto(("CreateJobObjectW", windll.kernel32),
                                       CreateJobObjectFlags)
CreateJobObject.errcheck = ErrCheckHandle

# AssignProcessToJobObject()

AssignProcessToJobObjectProto = WINFUNCTYPE(BOOL,      # Return type
                                            HANDLE,    # hJob
                                            HANDLE     # hProcess
                                            )
AssignProcessToJobObjectFlags = ((1, "hJob"),
                                 (1, "hProcess"))
AssignProcessToJobObject = AssignProcessToJobObjectProto(
    ("AssignProcessToJobObject", windll.kernel32),
    AssignProcessToJobObjectFlags)
AssignProcessToJobObject.errcheck = ErrCheckBool

# ResumeThread()

def ErrCheckResumeThread(result, func, args):
    if result == -1:
        raise WinError()

    return args

ResumeThreadProto = WINFUNCTYPE(DWORD,      # Return type
                                HANDLE      # hThread
                                )
ResumeThreadFlags = ((1, "hThread"),)
ResumeThread = ResumeThreadProto(("ResumeThread", windll.kernel32),
                                 ResumeThreadFlags)
ResumeThread.errcheck = ErrCheckResumeThread

# TerminateJobObject()

TerminateJobObjectProto = WINFUNCTYPE(BOOL,   # Return type
                                      HANDLE, # hJob
                                      UINT    # uExitCode
                                      )
TerminateJobObjectFlags = ((1, "hJob"),
                           (1, "uExitCode", 127))
TerminateJobObject = TerminateJobObjectProto(
    ("TerminateJobObject", windll.kernel32),
    TerminateJobObjectFlags)
TerminateJobObject.errcheck = ErrCheckBool

# WaitForSingleObject()

WaitForSingleObjectProto = WINFUNCTYPE(DWORD,  # Return type
                                       HANDLE, # hHandle
                                       DWORD,  # dwMilliseconds
                                       )
WaitForSingleObjectFlags = ((1, "hHandle"),
                            (1, "dwMilliseconds", -1))
WaitForSingleObject = WaitForSingleObjectProto(
    ("WaitForSingleObject", windll.kernel32),
    WaitForSingleObjectFlags)

INFINITE = -1
WAIT_TIMEOUT = 0x0102
WAIT_OBJECT_0 = 0x0
WAIT_ABANDONED = 0x0080

# GetExitCodeProcess()

GetExitCodeProcessProto = WINFUNCTYPE(BOOL,    # Return type
                                      HANDLE,  # hProcess
                                      LPDWORD, # lpExitCode
                                      )
GetExitCodeProcessFlags = ((1, "hProcess"),
                           (2, "lpExitCode"))
GetExitCodeProcess = GetExitCodeProcessProto(
    ("GetExitCodeProcess", windll.kernel32),
    GetExitCodeProcessFlags)
GetExitCodeProcess.errcheck = ErrCheckBool




#!/usr/bin/env python
# -*- coding: utf-8 -*-

# 
# Copyright 2013 Glencoe Software, Inc. All rights reserved.
# Use is subject to license terms supplied in LICENSE.txt

"""
py.test fixtures for gatewaytest
"""

from omero.gateway.scripts.testdb_create import TestDBHelper, dbhelpers


import pytest


class GatewayWrapper (TestDBHelper):

    def __init__(self):
        super(GatewayWrapper, self).__init__()
        self.setUp(skipTestDB=False, skipTestImages=True)

    def createTestImg_generated(self):
        ds = self.getTestDataset()
        assert ds
        testimg = self.createTestImage(dataset=ds)
        return testimg


@pytest.fixture(scope='class')
def gatewaywrapper(request):
    """
    Returns a test helper gateway object.
    """
    g = GatewayWrapper()

    def fin():
        g.tearDown()
        dbhelpers.cleanup()
    request.addfinalizer(fin)
    return g


@pytest.fixture(scope='function')
def author_testimg_generated(request, gatewaywrapper):
    """
    logs in as Author and returns the test image, creating it first if needed.
    """
    gatewaywrapper.loginAsAuthor()
    rv = gatewaywrapper.createTestImg_generated()
    return rv


@pytest.fixture(scope='function')
def author_testimg_tiny(request, gatewaywrapper):
    """
    logs in as Author and returns the test image, creating it first if needed.
    """
    gatewaywrapper.loginAsAuthor()
    rv = gatewaywrapper.getTinyTestImage(autocreate=True)
    return rv


@pytest.fixture(scope='function')
def author_testimg_tiny2(request, gatewaywrapper):
    """
    logs in as Author and returns the test image, creating it first if needed.
    """
    gatewaywrapper.loginAsAuthor()
    rv = gatewaywrapper.getTinyTestImage2(autocreate=True)
    return rv


@pytest.fixture(scope='function')
def author_testimg(request, gatewaywrapper):
    """
    logs in as Author and returns the test image, creating it first if needed.
    """
    gatewaywrapper.loginAsAuthor()
    rv = gatewaywrapper.getTestImage(autocreate=True)
    return rv


@pytest.fixture(scope='function')
def author_testimg_bad(request, gatewaywrapper):
    """
    logs in as Author and returns the test image, creating it first if needed.
    """
    gatewaywrapper.loginAsAuthor()
    rv = gatewaywrapper.getBadTestImage(autocreate=True)
    return rv


@pytest.fixture(scope='function')
def author_testimg_big(request, gatewaywrapper):
    """
    logs in as Author and returns the test image, creating it first if needed.
    """
    gatewaywrapper.loginAsAuthor()
    rv = gatewaywrapper.getBigTestImage(autocreate=True)
    return rv


@pytest.fixture(scope='function')
def author_testimg_32float(request, gatewaywrapper):
    """
    logs in as Author and returns the float image, creating it first if needed.
    """
    gatewaywrapper.loginAsAuthor()
    rv = gatewaywrapper.get32FloatTestImage(autocreate=True)
    return rv



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (C) 2006-2009 University of Dundee. All rights reserved.
#
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

# @author  William Moore &nbsp;&nbsp;&nbsp;&nbsp;
# <a href="mailto:will@lifesci.dundee.ac.uk">will@lifesci.dundee.ac.uk</a>
# @author  Jean-Marie Burel &nbsp;&nbsp;&nbsp;&nbsp;
# <a href="mailto:j.burel@dundee.ac.uk">j.burel@dundee.ac.uk</a>
# @author Donald MacDonald &nbsp;&nbsp;&nbsp;&nbsp;
# <a href="mailto:donald@lifesci.dundee.ac.uk">donald@lifesci.dundee.ac.uk</a>

"""
A collection of utility methods used by Figure scripts for producing
publication type of figures.
"""
from __future__ import division

from builtins import str
from builtins import range
from past.utils import old_div
try:
    from PIL import Image, ImageDraw  # see ticket:2597
except ImportError:
    import Image
    import ImageDraw

WHITE = (255, 255, 255)

SECS_MILLIS = "SECS_MILLIS"
SECS = "SECS"
MINS = "MINS"
HOURS = "HOURS"
MINS_SECS = "MINS_SECS"
HOURS_MINS = "HOURS_MINS"
HOURS_MINS_SECS = "HOURS_MINS_SECS"
HOURS_MINS_SECS_MILLIS = "HOURS_MINS_SECS_MILLIS"
TIME_UNITS = [SECS_MILLIS, SECS, MINS, HOURS, MINS_SECS,
              HOURS_MINS, HOURS_MINS_SECS, HOURS_MINS_SECS_MILLIS]


def getDatasetsProjectsFromImages(queryService, imageIds):
    """
    Query returns a map where each key is the imageId and the value is a list
    of (projectName, datasetName) tuples.
    If the image does not have a Dataset AND Project, the map will hold an
    empty list for that imageId.

    @param queryService:     The Omero query service
    @param imageIds:        A list of image IDs. [long]
    @return:            A map imageId:[(projectName, datasetName)]
    """
    ids = ",".join([str(i) for i in imageIds])

    query_string = "select i from Image i join fetch i.datasetLinks idl join " \
                   "fetch idl.parent d join fetch d.projectLinks pl join " \
                   "fetch pl.parent where i.id in (%s)" % ids

    images = queryService.findAllByQuery(query_string, None)
    results = {}

    for i in images:    # order of images not same as imageIds
        pdList = []
        imageId = i.getId().getValue()
        for link in i.iterateDatasetLinks():
            dataset = link.parent
            dName = dataset.getName().getValue()
            if dataset.sizeOfProjectLinks() == 0:
                pdList.append(("", dName))
            for dpLink in dataset.iterateProjectLinks():
                project = dpLink.parent
                pName = project.getName().getValue()
                pdList.append((pName, dName))
        results[imageId] = pdList

    # make sure the map contains all the imageIds
    for iId in imageIds:
        if iId not in results:
            results[iId] = []
    return results


def getTagsFromImages(metadataService, imageIds):
    """
    Query returns a map of key = imageId, value = [tagNames] for the image

    @param metadataService:        The Omero metadata service
    @param imageIds:            A list of image IDs. [long]
    @return:                A map of imageId:[tagName]
    """

    types = ["ome.model.annotations.TagAnnotation"]
    annotations = metadataService.loadAnnotations(
        "Image", imageIds, types, None, None)

    tagsMap = {}
    for i in imageIds:
        annots = annotations[i]
        tags = [a.getTextValue().getValue() for a in annots]
        tagsMap[i] = tags
    return tagsMap


def getTimes(queryService, pixelsId, tIndexes, theZ=None, theC=None):
    """
    Get the time in seconds (float) for the first plane (C = 0 & Z = 0) at
    each time-point for the defined pixels.
    Returns a map of tIndex: timeInSecs

    @param queryService:    The Omero queryService
    @param pixelsId:        The ID of the pixels object. long
    @param tIndexes:        List of time indexes. [int]
    @param theZ:        The Z plane index. Default is 0
    @param theC:        The Channel index. Default is 0
    @return:            A map of tIndex: timeInSecs
    """
    if theZ is None:
        theZ = 0
    if theC is None:
        theC = 0
    indexes = ",".join([str(t) for t in tIndexes])
    query = "from PlaneInfo as Info where Info.theT in (%s) and Info.theZ "\
            "in (%d) and Info.theC in (%d) and pixels.id='%d'" \
            % (indexes, theZ, theC, pixelsId)
    infoList = queryService.findAllByQuery(query, None)
    timeMap = {}
    for info in infoList:
        tIndex = info.theT.getValue()
        time = info.deltaT.getValue()
        timeMap[tIndex] = time
    return timeMap


def formatTime(seconds, timeUnits):
    """
    Returns a string formatting of the time (in seconds)
    according to the chosen timeUnits:
    "SECS_MILLIS", "SECS", "MINS", "HOURS", "MINS_SECS",
    "HOURS_MINS", "HOURS_MINS_SECS", "HOURS_MINS_SECS_MILLIS"

    @param seconds:     Time in seconds. float or int
    @param timeUnits:   A string denoting the format. One of the choices above.
    @return:            A string, such as "10" or "3:20"
    """
    neg = False
    if seconds < 0:
        seconds = seconds * -1
        neg = True
    label = None
    if timeUnits == "SECS_MILLIS":
        label = "%.2f" % seconds
    elif timeUnits == "SECS":
        label = "%d" % int(round(seconds))
    elif timeUnits == "MINS":
        mins = old_div(float(seconds), float(60))
        label = "%d" % int(round(mins))
    elif timeUnits == "HOURS":
        hrs = old_div(float(seconds), float(3600))
        label = "%d" % int(round(hrs))
    elif timeUnits == "MINS_SECS":
        mins = old_div(seconds, 60)
        secs = round(seconds % 60)
        label = "%d:%02d" % (mins, secs)
    elif timeUnits == "HOURS_MINS":
        hrs = old_div(seconds, 3600)
        mins = round(old_div((seconds % 3600), 60))
        label = "%d:%02d" % (hrs, mins)
    elif timeUnits == "HOURS_MINS_SECS":
        hrs = old_div(seconds, 3600)
        mins = old_div((seconds % 3600), 60)
        secs = round(seconds % (3600 * 60))
        label = "%d:%02d:%02d" % (hrs, mins, secs)
    elif timeUnits == "HOURS_MINS_SECS_MILLIS":
        hrs = old_div(seconds, 3600)
        mins = old_div((seconds % 3600), 60)
        secs = (seconds % (3600 * 60))
        label = "%d:%02d:%05.2f" % (hrs, mins, secs)
    else:
        label = "%.2f sec" % seconds
    return neg and "-%s" % label or label


def getTimeLabels(queryService, pixelsId, tIndexes, sizeT,
                  timeUnits=None, showRoiDuration=False):
    """
    Returns a list of time labels e.g. "10", "20" for the first plane at
    each t-index (C=0 and Z=0). If no planeInfo is available,
    returns plane number/total e.g "3/10"
    If time units are not specified, the most suitable units are chosen
    based on the max time.
    The list of label returned includes the timeUnits as the last string
    in the list, in case you didn't specify it.

    :param queryService:        The Omero query service
    :param pixelsId:            The ID of the pixels you want info for
    :param tIndexes:            List of t-index to get the times for.
                                Assumed to be in t order.
    :param sizeT:               The T dimension size of the pixels.
                                Used if no plane info
    :param timeUnits:           Format choice, see formatTime(). String
    :param showRoiDuration:     If true, times shown are from the start of
                                the ROI frames, otherwise use movie timestamp.
    :return:                    A list of strings, ordered same as tIndexes
    """
    secondsMap = getTimes(queryService, pixelsId, tIndexes)

    if timeUnits is None and len(secondsMap) > 0:
        maxSecs = max(secondsMap.values())
        if maxSecs > 3600:
            timeUnits = HOURS_MINS
        elif maxSecs > 60:
            timeUnits = MINS_SECS
        else:
            timeUnits = SECS_MILLIS

    labels = []
    for t in tIndexes:
        if t in secondsMap:
            seconds = secondsMap[t]
            if showRoiDuration:
                seconds = seconds - secondsMap[tIndexes[0]]
            labels.append(formatTime(seconds, timeUnits))
        else:
            labels.append("%d/%d" % (t + 1, sizeT))

    labels.append(timeUnits)
    return labels


def addScalebar(scalebar, xIndent, yIndent, image, pixels, colour):
    """
    Adds a scalebar at the bottom right of an image, No text.

    :param scalebar     length of scalebar in microns
    :param xIndent      indent from the right of the image
    :param yIndent      indent from the bottom of the image
    :param image        the PIL image to add scalebar to
    :param pixels       the pixels object
    :param colour       colour of the overlay as r,g,b tuple
    """
    draw = ImageDraw.Draw(image)
    if pixels.getPhysicalSizeX() is None:
        return False, "  Failed to add scale bar: Pixel size not defined."
    pixelSizeX = pixels.getPhysicalSizeX().getValue()
    if pixelSizeX <= 0:
        return False, "  Failed to add scale bar: Pixel size not defined."
    iWidth, iHeight = image.size
    lineThickness = (iHeight // 100) + 1
    scaleBarY = iHeight - yIndent
    scaleBarX = iWidth - scalebar // pixelSizeX - xIndent
    scaleBarX2 = iWidth - xIndent
    if (scaleBarX <= 0 or scaleBarX2 <= 0
            or scaleBarY <= 0 or scaleBarX2 > iWidth):
        return False, "  Failed to add scale bar: Scale bar is too large."
    for l in range(lineThickness):
        draw.line(
            [(scaleBarX, scaleBarY), (scaleBarX2, scaleBarY)], fill=colour)
        scaleBarY -= 1
    return True,  "  Scalebar added to the image."


def getVerticalLabels(labels, font, textGap):
    """
    Returns an image with the labels written vertically with the given font,
    black on white background
    """

    maxWidth = 0
    height = 0
    textHeight = font.getsize("testq")[1]
    for label in labels:
        maxWidth = max(maxWidth, font.getsize(label)[0])
        if height > 0:
            height += textGap
        height += textHeight
    size = (maxWidth, height)
    textCanvas = Image.new("RGB", size, WHITE)
    textdraw = ImageDraw.Draw(textCanvas)
    py = 0
    for label in labels:
        indent = old_div((maxWidth - font.getsize(label)[0]), 2)
        textdraw.text((indent, py), label, font=font, fill=(0, 0, 0))
        py += textHeight + textGap
    return textCanvas.rotate(90)



# -*- coding: utf-8 -*-
# **********************************************************************
#
# Copyright (c) 2003-2017 ZeroC, Inc. All rights reserved.
#
# This copy of Ice is licensed to you under the terms described in the
# ICE_LICENSE file included in this distribution.
#
# **********************************************************************
#
# Ice version 3.6.4
#
# <auto-generated>
#
# Generated from file `Dataset.ice'
#
# Warning: do not edit this file.
#
# </auto-generated>
#

from sys import version_info as _version_info_
import Ice, IcePy
import omero_model_IObject_ice
import omero_RTypes_ice
import omero_model_RTypes_ice
import omero_System_ice
import omero_Collections_ice

# Included module omero
_M_omero = Ice.openModule('omero')

# Included module omero.model
_M_omero.model = Ice.openModule('omero.model')

# Included module Ice
_M_Ice = Ice.openModule('Ice')

# Included module omero.sys
_M_omero.sys = Ice.openModule('omero.sys')

# Included module omero.api
_M_omero.api = Ice.openModule('omero.api')

# Start of module omero
__name__ = 'omero'

# Start of module omero.model
__name__ = 'omero.model'

if 'ProjectDatasetLink' not in _M_omero.model.__dict__:
    _M_omero.model._t_ProjectDatasetLink = IcePy.declareClass('::omero::model::ProjectDatasetLink')
    _M_omero.model._t_ProjectDatasetLinkPrx = IcePy.declareProxy('::omero::model::ProjectDatasetLink')

if 'Project' not in _M_omero.model.__dict__:
    _M_omero.model._t_Project = IcePy.declareClass('::omero::model::Project')
    _M_omero.model._t_ProjectPrx = IcePy.declareProxy('::omero::model::Project')

if 'DatasetImageLink' not in _M_omero.model.__dict__:
    _M_omero.model._t_DatasetImageLink = IcePy.declareClass('::omero::model::DatasetImageLink')
    _M_omero.model._t_DatasetImageLinkPrx = IcePy.declareProxy('::omero::model::DatasetImageLink')

if 'Image' not in _M_omero.model.__dict__:
    _M_omero.model._t_Image = IcePy.declareClass('::omero::model::Image')
    _M_omero.model._t_ImagePrx = IcePy.declareProxy('::omero::model::Image')

if 'DatasetAnnotationLink' not in _M_omero.model.__dict__:
    _M_omero.model._t_DatasetAnnotationLink = IcePy.declareClass('::omero::model::DatasetAnnotationLink')
    _M_omero.model._t_DatasetAnnotationLinkPrx = IcePy.declareProxy('::omero::model::DatasetAnnotationLink')

if 'Annotation' not in _M_omero.model.__dict__:
    _M_omero.model._t_Annotation = IcePy.declareClass('::omero::model::Annotation')
    _M_omero.model._t_AnnotationPrx = IcePy.declareProxy('::omero::model::Annotation')

if 'Details' not in _M_omero.model.__dict__:
    _M_omero.model._t_Details = IcePy.declareClass('::omero::model::Details')
    _M_omero.model._t_DetailsPrx = IcePy.declareProxy('::omero::model::Details')

if '_t_DatasetProjectLinksSeq' not in _M_omero.model.__dict__:
    _M_omero.model._t_DatasetProjectLinksSeq = IcePy.defineSequence('::omero::model::DatasetProjectLinksSeq', (), _M_omero.model._t_ProjectDatasetLink)

if '_t_DatasetLinkedProjectSeq' not in _M_omero.model.__dict__:
    _M_omero.model._t_DatasetLinkedProjectSeq = IcePy.defineSequence('::omero::model::DatasetLinkedProjectSeq', (), _M_omero.model._t_Project)

if '_t_DatasetImageLinksSeq' not in _M_omero.model.__dict__:
    _M_omero.model._t_DatasetImageLinksSeq = IcePy.defineSequence('::omero::model::DatasetImageLinksSeq', (), _M_omero.model._t_DatasetImageLink)

if '_t_DatasetLinkedImageSeq' not in _M_omero.model.__dict__:
    _M_omero.model._t_DatasetLinkedImageSeq = IcePy.defineSequence('::omero::model::DatasetLinkedImageSeq', (), _M_omero.model._t_Image)

if '_t_DatasetAnnotationLinksSeq' not in _M_omero.model.__dict__:
    _M_omero.model._t_DatasetAnnotationLinksSeq = IcePy.defineSequence('::omero::model::DatasetAnnotationLinksSeq', (), _M_omero.model._t_DatasetAnnotationLink)

if '_t_DatasetLinkedAnnotationSeq' not in _M_omero.model.__dict__:
    _M_omero.model._t_DatasetLinkedAnnotationSeq = IcePy.defineSequence('::omero::model::DatasetLinkedAnnotationSeq', (), _M_omero.model._t_Annotation)

if 'Dataset' not in _M_omero.model.__dict__:
    _M_omero.model.Dataset = Ice.createTempClass()
    class Dataset(_M_omero.model.IObject):
        def __init__(self, _id=None, _details=None, _loaded=False, _version=None, _projectLinksSeq=None, _projectLinksLoaded=False, _projectLinksCountPerOwner=None, _imageLinksSeq=None, _imageLinksLoaded=False, _imageLinksCountPerOwner=None, _annotationLinksSeq=None, _annotationLinksLoaded=False, _annotationLinksCountPerOwner=None, _name=None, _description=None):
            if Ice.getType(self) == _M_omero.model.Dataset:
                raise RuntimeError('omero.model.Dataset is an abstract class')
            _M_omero.model.IObject.__init__(self, _id, _details, _loaded)
            self._version = _version
            self._projectLinksSeq = _projectLinksSeq
            self._projectLinksLoaded = _projectLinksLoaded
            self._projectLinksCountPerOwner = _projectLinksCountPerOwner
            self._imageLinksSeq = _imageLinksSeq
            self._imageLinksLoaded = _imageLinksLoaded
            self._imageLinksCountPerOwner = _imageLinksCountPerOwner
            self._annotationLinksSeq = _annotationLinksSeq
            self._annotationLinksLoaded = _annotationLinksLoaded
            self._annotationLinksCountPerOwner = _annotationLinksCountPerOwner
            self._name = _name
            self._description = _description

        def ice_ids(self, current=None):
            return ('::Ice::Object', '::omero::model::Dataset', '::omero::model::IObject')

        def ice_id(self, current=None):
            return '::omero::model::Dataset'

        def ice_staticId():
            return '::omero::model::Dataset'
        ice_staticId = staticmethod(ice_staticId)

        def getVersion(self, current=None):
            pass

        def setVersion(self, theVersion, current=None):
            pass

        def unloadProjectLinks(self, current=None):
            pass

        def sizeOfProjectLinks(self, current=None):
            pass

        def copyProjectLinks(self, current=None):
            pass

        def addProjectDatasetLink(self, target, current=None):
            pass

        def addAllProjectDatasetLinkSet(self, targets, current=None):
            pass

        def removeProjectDatasetLink(self, theTarget, current=None):
            pass

        def removeAllProjectDatasetLinkSet(self, targets, current=None):
            pass

        def clearProjectLinks(self, current=None):
            pass

        def reloadProjectLinks(self, toCopy, current=None):
            pass

        def getProjectLinksCountPerOwner(self, current=None):
            pass

        def linkProject(self, addition, current=None):
            pass

        def addProjectDatasetLinkToBoth(self, link, bothSides, current=None):
            pass

        def findProjectDatasetLink(self, removal, current=None):
            pass

        def unlinkProject(self, removal, current=None):
            pass

        def removeProjectDatasetLinkFromBoth(self, link, bothSides, current=None):
            pass

        def linkedProjectList(self, current=None):
            pass

        def unloadImageLinks(self, current=None):
            pass

        def sizeOfImageLinks(self, current=None):
            pass

        def copyImageLinks(self, current=None):
            pass

        def addDatasetImageLink(self, target, current=None):
            pass

        def addAllDatasetImageLinkSet(self, targets, current=None):
            pass

        def removeDatasetImageLink(self, theTarget, current=None):
            pass

        def removeAllDatasetImageLinkSet(self, targets, current=None):
            pass

        def clearImageLinks(self, current=None):
            pass

        def reloadImageLinks(self, toCopy, current=None):
            pass

        def getImageLinksCountPerOwner(self, current=None):
            pass

        def linkImage(self, addition, current=None):
            pass

        def addDatasetImageLinkToBoth(self, link, bothSides, current=None):
            pass

        def findDatasetImageLink(self, removal, current=None):
            pass

        def unlinkImage(self, removal, current=None):
            pass

        def removeDatasetImageLinkFromBoth(self, link, bothSides, current=None):
            pass

        def linkedImageList(self, current=None):
            pass

        def unloadAnnotationLinks(self, current=None):
            pass

        def sizeOfAnnotationLinks(self, current=None):
            pass

        def copyAnnotationLinks(self, current=None):
            pass

        def addDatasetAnnotationLink(self, target, current=None):
            pass

        def addAllDatasetAnnotationLinkSet(self, targets, current=None):
            pass

        def removeDatasetAnnotationLink(self, theTarget, current=None):
            pass

        def removeAllDatasetAnnotationLinkSet(self, targets, current=None):
            pass

        def clearAnnotationLinks(self, current=None):
            pass

        def reloadAnnotationLinks(self, toCopy, current=None):
            pass

        def getAnnotationLinksCountPerOwner(self, current=None):
            pass

        def linkAnnotation(self, addition, current=None):
            pass

        def addDatasetAnnotationLinkToBoth(self, link, bothSides, current=None):
            pass

        def findDatasetAnnotationLink(self, removal, current=None):
            pass

        def unlinkAnnotation(self, removal, current=None):
            pass

        def removeDatasetAnnotationLinkFromBoth(self, link, bothSides, current=None):
            pass

        def linkedAnnotationList(self, current=None):
            pass

        def getName(self, current=None):
            pass

        def setName(self, theName, current=None):
            pass

        def getDescription(self, current=None):
            pass

        def setDescription(self, theDescription, current=None):
            pass

        def __str__(self):
            return IcePy.stringify(self, _M_omero.model._t_Dataset)

        __repr__ = __str__

    _M_omero.model.DatasetPrx = Ice.createTempClass()
    class DatasetPrx(_M_omero.model.IObjectPrx):

        def getVersion(self, _ctx=None):
            return _M_omero.model.Dataset._op_getVersion.invoke(self, ((), _ctx))

        def begin_getVersion(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Dataset._op_getVersion.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getVersion(self, _r):
            return _M_omero.model.Dataset._op_getVersion.end(self, _r)

        def setVersion(self, theVersion, _ctx=None):
            return _M_omero.model.Dataset._op_setVersion.invoke(self, ((theVersion, ), _ctx))

        def begin_setVersion(self, theVersion, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Dataset._op_setVersion.begin(self, ((theVersion, ), _response, _ex, _sent, _ctx))

        def end_setVersion(self, _r):
            return _M_omero.model.Dataset._op_setVersion.end(self, _r)

        def unloadProjectLinks(self, _ctx=None):
            return _M_omero.model.Dataset._op_unloadProjectLinks.invoke(self, ((), _ctx))

        def begin_unloadProjectLinks(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Dataset._op_unloadProjectLinks.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_unloadProjectLinks(self, _r):
            return _M_omero.model.Dataset._op_unloadProjectLinks.end(self, _r)

        def sizeOfProjectLinks(self, _ctx=None):
            return _M_omero.model.Dataset._op_sizeOfProjectLinks.invoke(self, ((), _ctx))

        def begin_sizeOfProjectLinks(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Dataset._op_sizeOfProjectLinks.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_sizeOfProjectLinks(self, _r):
            return _M_omero.model.Dataset._op_sizeOfProjectLinks.end(self, _r)

        def copyProjectLinks(self, _ctx=None):
            return _M_omero.model.Dataset._op_copyProjectLinks.invoke(self, ((), _ctx))

        def begin_copyProjectLinks(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Dataset._op_copyProjectLinks.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_copyProjectLinks(self, _r):
            return _M_omero.model.Dataset._op_copyProjectLinks.end(self, _r)

        def addProjectDatasetLink(self, target, _ctx=None):
            return _M_omero.model.Dataset._op_addProjectDatasetLink.invoke(self, ((target, ), _ctx))

        def begin_addProjectDatasetLink(self, target, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Dataset._op_addProjectDatasetLink.begin(self, ((target, ), _response, _ex, _sent, _ctx))

        def end_addProjectDatasetLink(self, _r):
            return _M_omero.model.Dataset._op_addProjectDatasetLink.end(self, _r)

        def addAllProjectDatasetLinkSet(self, targets, _ctx=None):
            return _M_omero.model.Dataset._op_addAllProjectDatasetLinkSet.invoke(self, ((targets, ), _ctx))

        def begin_addAllProjectDatasetLinkSet(self, targets, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Dataset._op_addAllProjectDatasetLinkSet.begin(self, ((targets, ), _response, _ex, _sent, _ctx))

        def end_addAllProjectDatasetLinkSet(self, _r):
            return _M_omero.model.Dataset._op_addAllProjectDatasetLinkSet.end(self, _r)

        def removeProjectDatasetLink(self, theTarget, _ctx=None):
            return _M_omero.model.Dataset._op_removeProjectDatasetLink.invoke(self, ((theTarget, ), _ctx))

        def begin_removeProjectDatasetLink(self, theTarget, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Dataset._op_removeProjectDatasetLink.begin(self, ((theTarget, ), _response, _ex, _sent, _ctx))

        def end_removeProjectDatasetLink(self, _r):
            return _M_omero.model.Dataset._op_removeProjectDatasetLink.end(self, _r)

        def removeAllProjectDatasetLinkSet(self, targets, _ctx=None):
            return _M_omero.model.Dataset._op_removeAllProjectDatasetLinkSet.invoke(self, ((targets, ), _ctx))

        def begin_removeAllProjectDatasetLinkSet(self, targets, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Dataset._op_removeAllProjectDatasetLinkSet.begin(self, ((targets, ), _response, _ex, _sent, _ctx))

        def end_removeAllProjectDatasetLinkSet(self, _r):
            return _M_omero.model.Dataset._op_removeAllProjectDatasetLinkSet.end(self, _r)

        def clearProjectLinks(self, _ctx=None):
            return _M_omero.model.Dataset._op_clearProjectLinks.invoke(self, ((), _ctx))

        def begin_clearProjectLinks(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Dataset._op_clearProjectLinks.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_clearProjectLinks(self, _r):
            return _M_omero.model.Dataset._op_clearProjectLinks.end(self, _r)

        def reloadProjectLinks(self, toCopy, _ctx=None):
            return _M_omero.model.Dataset._op_reloadProjectLinks.invoke(self, ((toCopy, ), _ctx))

        def begin_reloadProjectLinks(self, toCopy, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Dataset._op_reloadProjectLinks.begin(self, ((toCopy, ), _response, _ex, _sent, _ctx))

        def end_reloadProjectLinks(self, _r):
            return _M_omero.model.Dataset._op_reloadProjectLinks.end(self, _r)

        def getProjectLinksCountPerOwner(self, _ctx=None):
            return _M_omero.model.Dataset._op_getProjectLinksCountPerOwner.invoke(self, ((), _ctx))

        def begin_getProjectLinksCountPerOwner(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Dataset._op_getProjectLinksCountPerOwner.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getProjectLinksCountPerOwner(self, _r):
            return _M_omero.model.Dataset._op_getProjectLinksCountPerOwner.end(self, _r)

        def linkProject(self, addition, _ctx=None):
            return _M_omero.model.Dataset._op_linkProject.invoke(self, ((addition, ), _ctx))

        def begin_linkProject(self, addition, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Dataset._op_linkProject.begin(self, ((addition, ), _response, _ex, _sent, _ctx))

        def end_linkProject(self, _r):
            return _M_omero.model.Dataset._op_linkProject.end(self, _r)

        def addProjectDatasetLinkToBoth(self, link, bothSides, _ctx=None):
            return _M_omero.model.Dataset._op_addProjectDatasetLinkToBoth.invoke(self, ((link, bothSides), _ctx))

        def begin_addProjectDatasetLinkToBoth(self, link, bothSides, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Dataset._op_addProjectDatasetLinkToBoth.begin(self, ((link, bothSides), _response, _ex, _sent, _ctx))

        def end_addProjectDatasetLinkToBoth(self, _r):
            return _M_omero.model.Dataset._op_addProjectDatasetLinkToBoth.end(self, _r)

        def findProjectDatasetLink(self, removal, _ctx=None):
            return _M_omero.model.Dataset._op_findProjectDatasetLink.invoke(self, ((removal, ), _ctx))

        def begin_findProjectDatasetLink(self, removal, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Dataset._op_findProjectDatasetLink.begin(self, ((removal, ), _response, _ex, _sent, _ctx))

        def end_findProjectDatasetLink(self, _r):
            return _M_omero.model.Dataset._op_findProjectDatasetLink.end(self, _r)

        def unlinkProject(self, removal, _ctx=None):
            return _M_omero.model.Dataset._op_unlinkProject.invoke(self, ((removal, ), _ctx))

        def begin_unlinkProject(self, removal, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Dataset._op_unlinkProject.begin(self, ((removal, ), _response, _ex, _sent, _ctx))

        def end_unlinkProject(self, _r):
            return _M_omero.model.Dataset._op_unlinkProject.end(self, _r)

        def removeProjectDatasetLinkFromBoth(self, link, bothSides, _ctx=None):
            return _M_omero.model.Dataset._op_removeProjectDatasetLinkFromBoth.invoke(self, ((link, bothSides), _ctx))

        def begin_removeProjectDatasetLinkFromBoth(self, link, bothSides, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Dataset._op_removeProjectDatasetLinkFromBoth.begin(self, ((link, bothSides), _response, _ex, _sent, _ctx))

        def end_removeProjectDatasetLinkFromBoth(self, _r):
            return _M_omero.model.Dataset._op_removeProjectDatasetLinkFromBoth.end(self, _r)

        def linkedProjectList(self, _ctx=None):
            return _M_omero.model.Dataset._op_linkedProjectList.invoke(self, ((), _ctx))

        def begin_linkedProjectList(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Dataset._op_linkedProjectList.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_linkedProjectList(self, _r):
            return _M_omero.model.Dataset._op_linkedProjectList.end(self, _r)

        def unloadImageLinks(self, _ctx=None):
            return _M_omero.model.Dataset._op_unloadImageLinks.invoke(self, ((), _ctx))

        def begin_unloadImageLinks(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Dataset._op_unloadImageLinks.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_unloadImageLinks(self, _r):
            return _M_omero.model.Dataset._op_unloadImageLinks.end(self, _r)

        def sizeOfImageLinks(self, _ctx=None):
            return _M_omero.model.Dataset._op_sizeOfImageLinks.invoke(self, ((), _ctx))

        def begin_sizeOfImageLinks(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Dataset._op_sizeOfImageLinks.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_sizeOfImageLinks(self, _r):
            return _M_omero.model.Dataset._op_sizeOfImageLinks.end(self, _r)

        def copyImageLinks(self, _ctx=None):
            return _M_omero.model.Dataset._op_copyImageLinks.invoke(self, ((), _ctx))

        def begin_copyImageLinks(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Dataset._op_copyImageLinks.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_copyImageLinks(self, _r):
            return _M_omero.model.Dataset._op_copyImageLinks.end(self, _r)

        def addDatasetImageLink(self, target, _ctx=None):
            return _M_omero.model.Dataset._op_addDatasetImageLink.invoke(self, ((target, ), _ctx))

        def begin_addDatasetImageLink(self, target, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Dataset._op_addDatasetImageLink.begin(self, ((target, ), _response, _ex, _sent, _ctx))

        def end_addDatasetImageLink(self, _r):
            return _M_omero.model.Dataset._op_addDatasetImageLink.end(self, _r)

        def addAllDatasetImageLinkSet(self, targets, _ctx=None):
            return _M_omero.model.Dataset._op_addAllDatasetImageLinkSet.invoke(self, ((targets, ), _ctx))

        def begin_addAllDatasetImageLinkSet(self, targets, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Dataset._op_addAllDatasetImageLinkSet.begin(self, ((targets, ), _response, _ex, _sent, _ctx))

        def end_addAllDatasetImageLinkSet(self, _r):
            return _M_omero.model.Dataset._op_addAllDatasetImageLinkSet.end(self, _r)

        def removeDatasetImageLink(self, theTarget, _ctx=None):
            return _M_omero.model.Dataset._op_removeDatasetImageLink.invoke(self, ((theTarget, ), _ctx))

        def begin_removeDatasetImageLink(self, theTarget, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Dataset._op_removeDatasetImageLink.begin(self, ((theTarget, ), _response, _ex, _sent, _ctx))

        def end_removeDatasetImageLink(self, _r):
            return _M_omero.model.Dataset._op_removeDatasetImageLink.end(self, _r)

        def removeAllDatasetImageLinkSet(self, targets, _ctx=None):
            return _M_omero.model.Dataset._op_removeAllDatasetImageLinkSet.invoke(self, ((targets, ), _ctx))

        def begin_removeAllDatasetImageLinkSet(self, targets, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Dataset._op_removeAllDatasetImageLinkSet.begin(self, ((targets, ), _response, _ex, _sent, _ctx))

        def end_removeAllDatasetImageLinkSet(self, _r):
            return _M_omero.model.Dataset._op_removeAllDatasetImageLinkSet.end(self, _r)

        def clearImageLinks(self, _ctx=None):
            return _M_omero.model.Dataset._op_clearImageLinks.invoke(self, ((), _ctx))

        def begin_clearImageLinks(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Dataset._op_clearImageLinks.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_clearImageLinks(self, _r):
            return _M_omero.model.Dataset._op_clearImageLinks.end(self, _r)

        def reloadImageLinks(self, toCopy, _ctx=None):
            return _M_omero.model.Dataset._op_reloadImageLinks.invoke(self, ((toCopy, ), _ctx))

        def begin_reloadImageLinks(self, toCopy, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Dataset._op_reloadImageLinks.begin(self, ((toCopy, ), _response, _ex, _sent, _ctx))

        def end_reloadImageLinks(self, _r):
            return _M_omero.model.Dataset._op_reloadImageLinks.end(self, _r)

        def getImageLinksCountPerOwner(self, _ctx=None):
            return _M_omero.model.Dataset._op_getImageLinksCountPerOwner.invoke(self, ((), _ctx))

        def begin_getImageLinksCountPerOwner(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Dataset._op_getImageLinksCountPerOwner.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getImageLinksCountPerOwner(self, _r):
            return _M_omero.model.Dataset._op_getImageLinksCountPerOwner.end(self, _r)

        def linkImage(self, addition, _ctx=None):
            return _M_omero.model.Dataset._op_linkImage.invoke(self, ((addition, ), _ctx))

        def begin_linkImage(self, addition, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Dataset._op_linkImage.begin(self, ((addition, ), _response, _ex, _sent, _ctx))

        def end_linkImage(self, _r):
            return _M_omero.model.Dataset._op_linkImage.end(self, _r)

        def addDatasetImageLinkToBoth(self, link, bothSides, _ctx=None):
            return _M_omero.model.Dataset._op_addDatasetImageLinkToBoth.invoke(self, ((link, bothSides), _ctx))

        def begin_addDatasetImageLinkToBoth(self, link, bothSides, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Dataset._op_addDatasetImageLinkToBoth.begin(self, ((link, bothSides), _response, _ex, _sent, _ctx))

        def end_addDatasetImageLinkToBoth(self, _r):
            return _M_omero.model.Dataset._op_addDatasetImageLinkToBoth.end(self, _r)

        def findDatasetImageLink(self, removal, _ctx=None):
            return _M_omero.model.Dataset._op_findDatasetImageLink.invoke(self, ((removal, ), _ctx))

        def begin_findDatasetImageLink(self, removal, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Dataset._op_findDatasetImageLink.begin(self, ((removal, ), _response, _ex, _sent, _ctx))

        def end_findDatasetImageLink(self, _r):
            return _M_omero.model.Dataset._op_findDatasetImageLink.end(self, _r)

        def unlinkImage(self, removal, _ctx=None):
            return _M_omero.model.Dataset._op_unlinkImage.invoke(self, ((removal, ), _ctx))

        def begin_unlinkImage(self, removal, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Dataset._op_unlinkImage.begin(self, ((removal, ), _response, _ex, _sent, _ctx))

        def end_unlinkImage(self, _r):
            return _M_omero.model.Dataset._op_unlinkImage.end(self, _r)

        def removeDatasetImageLinkFromBoth(self, link, bothSides, _ctx=None):
            return _M_omero.model.Dataset._op_removeDatasetImageLinkFromBoth.invoke(self, ((link, bothSides), _ctx))

        def begin_removeDatasetImageLinkFromBoth(self, link, bothSides, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Dataset._op_removeDatasetImageLinkFromBoth.begin(self, ((link, bothSides), _response, _ex, _sent, _ctx))

        def end_removeDatasetImageLinkFromBoth(self, _r):
            return _M_omero.model.Dataset._op_removeDatasetImageLinkFromBoth.end(self, _r)

        def linkedImageList(self, _ctx=None):
            return _M_omero.model.Dataset._op_linkedImageList.invoke(self, ((), _ctx))

        def begin_linkedImageList(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Dataset._op_linkedImageList.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_linkedImageList(self, _r):
            return _M_omero.model.Dataset._op_linkedImageList.end(self, _r)

        def unloadAnnotationLinks(self, _ctx=None):
            return _M_omero.model.Dataset._op_unloadAnnotationLinks.invoke(self, ((), _ctx))

        def begin_unloadAnnotationLinks(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Dataset._op_unloadAnnotationLinks.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_unloadAnnotationLinks(self, _r):
            return _M_omero.model.Dataset._op_unloadAnnotationLinks.end(self, _r)

        def sizeOfAnnotationLinks(self, _ctx=None):
            return _M_omero.model.Dataset._op_sizeOfAnnotationLinks.invoke(self, ((), _ctx))

        def begin_sizeOfAnnotationLinks(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Dataset._op_sizeOfAnnotationLinks.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_sizeOfAnnotationLinks(self, _r):
            return _M_omero.model.Dataset._op_sizeOfAnnotationLinks.end(self, _r)

        def copyAnnotationLinks(self, _ctx=None):
            return _M_omero.model.Dataset._op_copyAnnotationLinks.invoke(self, ((), _ctx))

        def begin_copyAnnotationLinks(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Dataset._op_copyAnnotationLinks.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_copyAnnotationLinks(self, _r):
            return _M_omero.model.Dataset._op_copyAnnotationLinks.end(self, _r)

        def addDatasetAnnotationLink(self, target, _ctx=None):
            return _M_omero.model.Dataset._op_addDatasetAnnotationLink.invoke(self, ((target, ), _ctx))

        def begin_addDatasetAnnotationLink(self, target, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Dataset._op_addDatasetAnnotationLink.begin(self, ((target, ), _response, _ex, _sent, _ctx))

        def end_addDatasetAnnotationLink(self, _r):
            return _M_omero.model.Dataset._op_addDatasetAnnotationLink.end(self, _r)

        def addAllDatasetAnnotationLinkSet(self, targets, _ctx=None):
            return _M_omero.model.Dataset._op_addAllDatasetAnnotationLinkSet.invoke(self, ((targets, ), _ctx))

        def begin_addAllDatasetAnnotationLinkSet(self, targets, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Dataset._op_addAllDatasetAnnotationLinkSet.begin(self, ((targets, ), _response, _ex, _sent, _ctx))

        def end_addAllDatasetAnnotationLinkSet(self, _r):
            return _M_omero.model.Dataset._op_addAllDatasetAnnotationLinkSet.end(self, _r)

        def removeDatasetAnnotationLink(self, theTarget, _ctx=None):
            return _M_omero.model.Dataset._op_removeDatasetAnnotationLink.invoke(self, ((theTarget, ), _ctx))

        def begin_removeDatasetAnnotationLink(self, theTarget, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Dataset._op_removeDatasetAnnotationLink.begin(self, ((theTarget, ), _response, _ex, _sent, _ctx))

        def end_removeDatasetAnnotationLink(self, _r):
            return _M_omero.model.Dataset._op_removeDatasetAnnotationLink.end(self, _r)

        def removeAllDatasetAnnotationLinkSet(self, targets, _ctx=None):
            return _M_omero.model.Dataset._op_removeAllDatasetAnnotationLinkSet.invoke(self, ((targets, ), _ctx))

        def begin_removeAllDatasetAnnotationLinkSet(self, targets, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Dataset._op_removeAllDatasetAnnotationLinkSet.begin(self, ((targets, ), _response, _ex, _sent, _ctx))

        def end_removeAllDatasetAnnotationLinkSet(self, _r):
            return _M_omero.model.Dataset._op_removeAllDatasetAnnotationLinkSet.end(self, _r)

        def clearAnnotationLinks(self, _ctx=None):
            return _M_omero.model.Dataset._op_clearAnnotationLinks.invoke(self, ((), _ctx))

        def begin_clearAnnotationLinks(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Dataset._op_clearAnnotationLinks.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_clearAnnotationLinks(self, _r):
            return _M_omero.model.Dataset._op_clearAnnotationLinks.end(self, _r)

        def reloadAnnotationLinks(self, toCopy, _ctx=None):
            return _M_omero.model.Dataset._op_reloadAnnotationLinks.invoke(self, ((toCopy, ), _ctx))

        def begin_reloadAnnotationLinks(self, toCopy, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Dataset._op_reloadAnnotationLinks.begin(self, ((toCopy, ), _response, _ex, _sent, _ctx))

        def end_reloadAnnotationLinks(self, _r):
            return _M_omero.model.Dataset._op_reloadAnnotationLinks.end(self, _r)

        def getAnnotationLinksCountPerOwner(self, _ctx=None):
            return _M_omero.model.Dataset._op_getAnnotationLinksCountPerOwner.invoke(self, ((), _ctx))

        def begin_getAnnotationLinksCountPerOwner(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Dataset._op_getAnnotationLinksCountPerOwner.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getAnnotationLinksCountPerOwner(self, _r):
            return _M_omero.model.Dataset._op_getAnnotationLinksCountPerOwner.end(self, _r)

        def linkAnnotation(self, addition, _ctx=None):
            return _M_omero.model.Dataset._op_linkAnnotation.invoke(self, ((addition, ), _ctx))

        def begin_linkAnnotation(self, addition, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Dataset._op_linkAnnotation.begin(self, ((addition, ), _response, _ex, _sent, _ctx))

        def end_linkAnnotation(self, _r):
            return _M_omero.model.Dataset._op_linkAnnotation.end(self, _r)

        def addDatasetAnnotationLinkToBoth(self, link, bothSides, _ctx=None):
            return _M_omero.model.Dataset._op_addDatasetAnnotationLinkToBoth.invoke(self, ((link, bothSides), _ctx))

        def begin_addDatasetAnnotationLinkToBoth(self, link, bothSides, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Dataset._op_addDatasetAnnotationLinkToBoth.begin(self, ((link, bothSides), _response, _ex, _sent, _ctx))

        def end_addDatasetAnnotationLinkToBoth(self, _r):
            return _M_omero.model.Dataset._op_addDatasetAnnotationLinkToBoth.end(self, _r)

        def findDatasetAnnotationLink(self, removal, _ctx=None):
            return _M_omero.model.Dataset._op_findDatasetAnnotationLink.invoke(self, ((removal, ), _ctx))

        def begin_findDatasetAnnotationLink(self, removal, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Dataset._op_findDatasetAnnotationLink.begin(self, ((removal, ), _response, _ex, _sent, _ctx))

        def end_findDatasetAnnotationLink(self, _r):
            return _M_omero.model.Dataset._op_findDatasetAnnotationLink.end(self, _r)

        def unlinkAnnotation(self, removal, _ctx=None):
            return _M_omero.model.Dataset._op_unlinkAnnotation.invoke(self, ((removal, ), _ctx))

        def begin_unlinkAnnotation(self, removal, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Dataset._op_unlinkAnnotation.begin(self, ((removal, ), _response, _ex, _sent, _ctx))

        def end_unlinkAnnotation(self, _r):
            return _M_omero.model.Dataset._op_unlinkAnnotation.end(self, _r)

        def removeDatasetAnnotationLinkFromBoth(self, link, bothSides, _ctx=None):
            return _M_omero.model.Dataset._op_removeDatasetAnnotationLinkFromBoth.invoke(self, ((link, bothSides), _ctx))

        def begin_removeDatasetAnnotationLinkFromBoth(self, link, bothSides, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Dataset._op_removeDatasetAnnotationLinkFromBoth.begin(self, ((link, bothSides), _response, _ex, _sent, _ctx))

        def end_removeDatasetAnnotationLinkFromBoth(self, _r):
            return _M_omero.model.Dataset._op_removeDatasetAnnotationLinkFromBoth.end(self, _r)

        def linkedAnnotationList(self, _ctx=None):
            return _M_omero.model.Dataset._op_linkedAnnotationList.invoke(self, ((), _ctx))

        def begin_linkedAnnotationList(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Dataset._op_linkedAnnotationList.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_linkedAnnotationList(self, _r):
            return _M_omero.model.Dataset._op_linkedAnnotationList.end(self, _r)

        def getName(self, _ctx=None):
            return _M_omero.model.Dataset._op_getName.invoke(self, ((), _ctx))

        def begin_getName(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Dataset._op_getName.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getName(self, _r):
            return _M_omero.model.Dataset._op_getName.end(self, _r)

        def setName(self, theName, _ctx=None):
            return _M_omero.model.Dataset._op_setName.invoke(self, ((theName, ), _ctx))

        def begin_setName(self, theName, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Dataset._op_setName.begin(self, ((theName, ), _response, _ex, _sent, _ctx))

        def end_setName(self, _r):
            return _M_omero.model.Dataset._op_setName.end(self, _r)

        def getDescription(self, _ctx=None):
            return _M_omero.model.Dataset._op_getDescription.invoke(self, ((), _ctx))

        def begin_getDescription(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Dataset._op_getDescription.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getDescription(self, _r):
            return _M_omero.model.Dataset._op_getDescription.end(self, _r)

        def setDescription(self, theDescription, _ctx=None):
            return _M_omero.model.Dataset._op_setDescription.invoke(self, ((theDescription, ), _ctx))

        def begin_setDescription(self, theDescription, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Dataset._op_setDescription.begin(self, ((theDescription, ), _response, _ex, _sent, _ctx))

        def end_setDescription(self, _r):
            return _M_omero.model.Dataset._op_setDescription.end(self, _r)

        def checkedCast(proxy, facetOrCtx=None, _ctx=None):
            return _M_omero.model.DatasetPrx.ice_checkedCast(proxy, '::omero::model::Dataset', facetOrCtx, _ctx)
        checkedCast = staticmethod(checkedCast)

        def uncheckedCast(proxy, facet=None):
            return _M_omero.model.DatasetPrx.ice_uncheckedCast(proxy, facet)
        uncheckedCast = staticmethod(uncheckedCast)

        def ice_staticId():
            return '::omero::model::Dataset'
        ice_staticId = staticmethod(ice_staticId)

    _M_omero.model._t_DatasetPrx = IcePy.defineProxy('::omero::model::Dataset', DatasetPrx)

    _M_omero.model._t_Dataset = IcePy.declareClass('::omero::model::Dataset')

    _M_omero.model._t_Dataset = IcePy.defineClass('::omero::model::Dataset', Dataset, -1, (), True, False, _M_omero.model._t_IObject, (), (
        ('_version', (), _M_omero._t_RInt, False, 0),
        ('_projectLinksSeq', (), _M_omero.model._t_DatasetProjectLinksSeq, False, 0),
        ('_projectLinksLoaded', (), IcePy._t_bool, False, 0),
        ('_projectLinksCountPerOwner', (), _M_omero.sys._t_CountMap, False, 0),
        ('_imageLinksSeq', (), _M_omero.model._t_DatasetImageLinksSeq, False, 0),
        ('_imageLinksLoaded', (), IcePy._t_bool, False, 0),
        ('_imageLinksCountPerOwner', (), _M_omero.sys._t_CountMap, False, 0),
        ('_annotationLinksSeq', (), _M_omero.model._t_DatasetAnnotationLinksSeq, False, 0),
        ('_annotationLinksLoaded', (), IcePy._t_bool, False, 0),
        ('_annotationLinksCountPerOwner', (), _M_omero.sys._t_CountMap, False, 0),
        ('_name', (), _M_omero._t_RString, False, 0),
        ('_description', (), _M_omero._t_RString, False, 0)
    ))
    Dataset._ice_type = _M_omero.model._t_Dataset

    Dataset._op_getVersion = IcePy.Operation('getVersion', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero._t_RInt, False, 0), ())
    Dataset._op_setVersion = IcePy.Operation('setVersion', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero._t_RInt, False, 0),), (), None, ())
    Dataset._op_unloadProjectLinks = IcePy.Operation('unloadProjectLinks', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), None, ())
    Dataset._op_sizeOfProjectLinks = IcePy.Operation('sizeOfProjectLinks', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), IcePy._t_int, False, 0), ())
    Dataset._op_copyProjectLinks = IcePy.Operation('copyProjectLinks', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.model._t_DatasetProjectLinksSeq, False, 0), ())
    Dataset._op_addProjectDatasetLink = IcePy.Operation('addProjectDatasetLink', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_ProjectDatasetLink, False, 0),), (), None, ())
    Dataset._op_addAllProjectDatasetLinkSet = IcePy.Operation('addAllProjectDatasetLinkSet', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_DatasetProjectLinksSeq, False, 0),), (), None, ())
    Dataset._op_removeProjectDatasetLink = IcePy.Operation('removeProjectDatasetLink', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_ProjectDatasetLink, False, 0),), (), None, ())
    Dataset._op_removeAllProjectDatasetLinkSet = IcePy.Operation('removeAllProjectDatasetLinkSet', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_DatasetProjectLinksSeq, False, 0),), (), None, ())
    Dataset._op_clearProjectLinks = IcePy.Operation('clearProjectLinks', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), None, ())
    Dataset._op_reloadProjectLinks = IcePy.Operation('reloadProjectLinks', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_Dataset, False, 0),), (), None, ())
    Dataset._op_getProjectLinksCountPerOwner = IcePy.Operation('getProjectLinksCountPerOwner', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.sys._t_CountMap, False, 0), ())
    Dataset._op_linkProject = IcePy.Operation('linkProject', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_Project, False, 0),), (), ((), _M_omero.model._t_ProjectDatasetLink, False, 0), ())
    Dataset._op_addProjectDatasetLinkToBoth = IcePy.Operation('addProjectDatasetLinkToBoth', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_ProjectDatasetLink, False, 0), ((), IcePy._t_bool, False, 0)), (), None, ())
    Dataset._op_findProjectDatasetLink = IcePy.Operation('findProjectDatasetLink', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_Project, False, 0),), (), ((), _M_omero.model._t_DatasetProjectLinksSeq, False, 0), ())
    Dataset._op_unlinkProject = IcePy.Operation('unlinkProject', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_Project, False, 0),), (), None, ())
    Dataset._op_removeProjectDatasetLinkFromBoth = IcePy.Operation('removeProjectDatasetLinkFromBoth', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_ProjectDatasetLink, False, 0), ((), IcePy._t_bool, False, 0)), (), None, ())
    Dataset._op_linkedProjectList = IcePy.Operation('linkedProjectList', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.model._t_DatasetLinkedProjectSeq, False, 0), ())
    Dataset._op_unloadImageLinks = IcePy.Operation('unloadImageLinks', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), None, ())
    Dataset._op_sizeOfImageLinks = IcePy.Operation('sizeOfImageLinks', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), IcePy._t_int, False, 0), ())
    Dataset._op_copyImageLinks = IcePy.Operation('copyImageLinks', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.model._t_DatasetImageLinksSeq, False, 0), ())
    Dataset._op_addDatasetImageLink = IcePy.Operation('addDatasetImageLink', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_DatasetImageLink, False, 0),), (), None, ())
    Dataset._op_addAllDatasetImageLinkSet = IcePy.Operation('addAllDatasetImageLinkSet', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_DatasetImageLinksSeq, False, 0),), (), None, ())
    Dataset._op_removeDatasetImageLink = IcePy.Operation('removeDatasetImageLink', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_DatasetImageLink, False, 0),), (), None, ())
    Dataset._op_removeAllDatasetImageLinkSet = IcePy.Operation('removeAllDatasetImageLinkSet', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_DatasetImageLinksSeq, False, 0),), (), None, ())
    Dataset._op_clearImageLinks = IcePy.Operation('clearImageLinks', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), None, ())
    Dataset._op_reloadImageLinks = IcePy.Operation('reloadImageLinks', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_Dataset, False, 0),), (), None, ())
    Dataset._op_getImageLinksCountPerOwner = IcePy.Operation('getImageLinksCountPerOwner', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.sys._t_CountMap, False, 0), ())
    Dataset._op_linkImage = IcePy.Operation('linkImage', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_Image, False, 0),), (), ((), _M_omero.model._t_DatasetImageLink, False, 0), ())
    Dataset._op_addDatasetImageLinkToBoth = IcePy.Operation('addDatasetImageLinkToBoth', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_DatasetImageLink, False, 0), ((), IcePy._t_bool, False, 0)), (), None, ())
    Dataset._op_findDatasetImageLink = IcePy.Operation('findDatasetImageLink', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_Image, False, 0),), (), ((), _M_omero.model._t_DatasetImageLinksSeq, False, 0), ())
    Dataset._op_unlinkImage = IcePy.Operation('unlinkImage', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_Image, False, 0),), (), None, ())
    Dataset._op_removeDatasetImageLinkFromBoth = IcePy.Operation('removeDatasetImageLinkFromBoth', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_DatasetImageLink, False, 0), ((), IcePy._t_bool, False, 0)), (), None, ())
    Dataset._op_linkedImageList = IcePy.Operation('linkedImageList', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.model._t_DatasetLinkedImageSeq, False, 0), ())
    Dataset._op_unloadAnnotationLinks = IcePy.Operation('unloadAnnotationLinks', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), None, ())
    Dataset._op_sizeOfAnnotationLinks = IcePy.Operation('sizeOfAnnotationLinks', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), IcePy._t_int, False, 0), ())
    Dataset._op_copyAnnotationLinks = IcePy.Operation('copyAnnotationLinks', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.model._t_DatasetAnnotationLinksSeq, False, 0), ())
    Dataset._op_addDatasetAnnotationLink = IcePy.Operation('addDatasetAnnotationLink', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_DatasetAnnotationLink, False, 0),), (), None, ())
    Dataset._op_addAllDatasetAnnotationLinkSet = IcePy.Operation('addAllDatasetAnnotationLinkSet', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_DatasetAnnotationLinksSeq, False, 0),), (), None, ())
    Dataset._op_removeDatasetAnnotationLink = IcePy.Operation('removeDatasetAnnotationLink', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_DatasetAnnotationLink, False, 0),), (), None, ())
    Dataset._op_removeAllDatasetAnnotationLinkSet = IcePy.Operation('removeAllDatasetAnnotationLinkSet', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_DatasetAnnotationLinksSeq, False, 0),), (), None, ())
    Dataset._op_clearAnnotationLinks = IcePy.Operation('clearAnnotationLinks', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), None, ())
    Dataset._op_reloadAnnotationLinks = IcePy.Operation('reloadAnnotationLinks', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_Dataset, False, 0),), (), None, ())
    Dataset._op_getAnnotationLinksCountPerOwner = IcePy.Operation('getAnnotationLinksCountPerOwner', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.sys._t_CountMap, False, 0), ())
    Dataset._op_linkAnnotation = IcePy.Operation('linkAnnotation', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_Annotation, False, 0),), (), ((), _M_omero.model._t_DatasetAnnotationLink, False, 0), ())
    Dataset._op_addDatasetAnnotationLinkToBoth = IcePy.Operation('addDatasetAnnotationLinkToBoth', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_DatasetAnnotationLink, False, 0), ((), IcePy._t_bool, False, 0)), (), None, ())
    Dataset._op_findDatasetAnnotationLink = IcePy.Operation('findDatasetAnnotationLink', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_Annotation, False, 0),), (), ((), _M_omero.model._t_DatasetAnnotationLinksSeq, False, 0), ())
    Dataset._op_unlinkAnnotation = IcePy.Operation('unlinkAnnotation', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_Annotation, False, 0),), (), None, ())
    Dataset._op_removeDatasetAnnotationLinkFromBoth = IcePy.Operation('removeDatasetAnnotationLinkFromBoth', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_DatasetAnnotationLink, False, 0), ((), IcePy._t_bool, False, 0)), (), None, ())
    Dataset._op_linkedAnnotationList = IcePy.Operation('linkedAnnotationList', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.model._t_DatasetLinkedAnnotationSeq, False, 0), ())
    Dataset._op_getName = IcePy.Operation('getName', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero._t_RString, False, 0), ())
    Dataset._op_setName = IcePy.Operation('setName', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero._t_RString, False, 0),), (), None, ())
    Dataset._op_getDescription = IcePy.Operation('getDescription', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero._t_RString, False, 0), ())
    Dataset._op_setDescription = IcePy.Operation('setDescription', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero._t_RString, False, 0),), (), None, ())

    _M_omero.model.Dataset = Dataset
    del Dataset

    _M_omero.model.DatasetPrx = DatasetPrx
    del DatasetPrx

# End of module omero.model

__name__ = 'omero'

# End of module omero



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# Copyright (C) 2008-2016 Glencoe Software, Inc. All Rights Reserved.
# Use is subject to license terms supplied in LICENSE.txt
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

"""
   Plugin for our managing the OMERO database.

   Plugin read by omero.cli.Cli during initialization. The method(s)
   defined here will be added to the Cli class for later use.
"""
from __future__ import division

from future.utils import bytes_to_native_str
from future.utils import isbytes
from past.utils import old_div
from omero.cli import BaseControl
from omero.cli import CLI

from argparse import FileType, SUPPRESS

from omero.install.windows_warning import windows_warning, WINDOWS_WARNING

from omero_ext.path import path

import omero.java
import platform
import sys
import time

if sys.version_info >= (3, 0, 0):
    # Keep str behavior on Python 2
    from builtins import str


HELP = """Database tools for creating scripts, setting passwords, etc."""


if platform.system() == 'Windows':
    HELP += ("\n\n%s" % WINDOWS_WARNING)


class DatabaseControl(BaseControl):

    def _configure(self, parser):
        sub = parser.sub()

        script = sub.add_parser(
            "script", help="Generates a DB creation script")
        script.set_defaults(func=self.script)
        try:
            # Python 3
            ft = FileType(mode="w", encoding='utf-8')
        except TypeError:
            ft = FileType(mode="w")
        script.add_argument(
            "-f", "--file", type=ft,
            help="Optional file to save to. Use '-' for stdout.")

        script.add_argument("posversion", nargs="?", help=SUPPRESS)
        script.add_argument("pospatch", nargs="?", help=SUPPRESS)
        script.add_argument("pospassword", nargs="?", help=SUPPRESS)

        script.add_argument("--version", help=SUPPRESS)
        script.add_argument("--patch", help=SUPPRESS)
        script.add_argument("--password", help="OMERO root password")

        pw = sub.add_parser(
            "password",
            help="Prints SQL command for updating your root password")
        pw.set_defaults(func=self.password)
        pw_spec = pw.add_mutually_exclusive_group()
        pw_spec.add_argument("password", nargs="?")
        pw_spec.add_argument("--empty", action="store_true",
                             help=("Remove the password, "
                                   "allowing any for login when guest."))
        pw.add_argument("--user-id",
                        help="User ID to salt into the password. "
                        "Defaults to '0', i.e. 'root'",
                        default="0")

        for x in (pw, script):
            x.add_argument(
                "--no-salt", action="store_true",
                help="Disable the salting of passwords")

    def _lookup(self, key, defaults, args):
        """
        Get a value from a flag arg, positional arg, or default properties
        """
        propname = "omero.db." + key
        vdef = defaults.properties.getProperty(propname)
        varg = getattr(args, key)
        vpos = getattr(args, 'pos' + key)
        if varg:
            if vpos:
                self.ctx.die(
                    1, "ERROR: Flag and positional argument given for %s" % key
                    )
            v = varg
        elif vpos:
            v = vpos
        elif vdef:
            v = vdef
        else:
            self.ctx.die(1, "No value found for %s" % propname)
        self.ctx.err("Using %s for %s" % (v, key))
        return v

    def _has_user_id(self, args):
        return args and "user_id" in args and args.user_id is not None

    def _get_password_hash(self, args, root_pass=None, old_prompt=False):

        prompt = " for OMERO "
        if self._has_user_id(args) and not old_prompt:
            prompt += "user %s" % args.user_id
        else:
            prompt += "root user"
        root_pass = self._ask_for_password(prompt, root_pass)

        jars = str(self.ctx.dir / "lib" / "server") + "/*"
        cmd = ["ome.security.auth.PasswordUtil", root_pass]
        if not args.no_salt and self._has_user_id(args):
            cmd.append(args.user_id)
        p = omero.java.popen(["-cp", jars] + cmd)
        rc = p.wait()
        if rc != 0:
            out, err = p.communicate()
            self.ctx.die(rc, "PasswordUtil failed: %s" % err.decode(
                errors='replace'))
        value = p.communicate()[0]
        if not value or len(value) == 0:
            self.ctx.die(100, "Encoded password is empty")
        return value.strip().decode()

    def _copy(self, input_path, output, func, cfg=None):
            try:
                input = open(str(input_path), encoding='utf-8')
            except TypeError:
                input = open(str(input_path))
            try:
                for s in input:
                        try:
                            if cfg:
                                output.write(func(s) % cfg)
                            else:
                                output.write(func(s))
                        except Exception as e:
                            self.ctx.dbg(str(e))
                            self.ctx.die(
                                154, "Failed to map line: %s\nError: %s"
                                % (s, e))
            finally:
                input.close()

    def _make_replace(self, root_pass, db_vers, db_patch):
        def fix(str_in):
            if isbytes(str_in):
                str_in = bytes_to_native_str(str_in)
            return str_in
        def replace_method(str_in):
            str_out = str_in.replace("@ROOTPASS@", fix(root_pass))
            str_out = str_out.replace("@DBVERSION@", fix(db_vers))
            str_out = str_out.replace("@DBPATCH@", fix(db_patch))
            return str_out
        return replace_method

    def _db_profile(self):
        from omero.install.config_parser import PropertyParser
        property_lines = self.ctx.get_config_property_lines(self.dir)
        for property in PropertyParser().parse_lines(property_lines):
            if property.key == 'omero.db.profile':
                return property.val
        raise KeyError('Configuration key not set: omero.db.profile')

    def _sql_directory(self, db_vers, db_patch):
        """
        See #2689
        """
        dbprofile = self._db_profile()
        sql_directory = self.ctx.dir / "sql" / dbprofile / \
            ("%s__%s" % (db_vers, db_patch))
        if not sql_directory.exists():
            self.ctx.die(2, "Invalid Database version/patch: %s does not"
                         " exist" % sql_directory)
        return sql_directory

    def _create(self, sql_directory, db_vers, db_patch, password_hash, args,
                location=None):
        sql_directory = self._sql_directory(db_vers, db_patch)
        if not sql_directory.exists():
            self.ctx.die(2, "Invalid Database version/patch: %s does not"
                         " exist" % sql_directory)

        if args and args.file:
            output = args.file
            script = "<filename here>"
        else:
            script = "%s__%s.sql" % (db_vers, db_patch)
            location = old_div(path.getcwd(), script)
            try:
                output = open(location, 'w', encoding='utf-8')
            except TypeError:
                output = open(location, 'w')
            self.ctx.out("Saving to " + location)

        try:
            dbprofile = self._db_profile()
            header = old_div(sql_directory, ("%s-header.sql" % dbprofile))
            footer = old_div(sql_directory, ("%s-footer.sql" % dbprofile))
            if header.exists():
                # 73 multiple DB support. OMERO 4.3+
                cfg = {
                    "TIME": time.ctime(time.time()),
                    "DIR": sql_directory,
                    "SCRIPT": script}
                self._copy(header, output, str, cfg)
                self._copy(old_div(sql_directory,"schema.sql"), output, str)
                self._copy(old_div(sql_directory,"views.sql"), output, str)
                self._copy(
                    footer, output,
                    self._make_replace(password_hash, db_vers, db_patch), cfg)
            else:
                # OMERO 4.2.x and before
                output.write("""
--
-- GENERATED %s from %s
--
-- This file was created by the `omero db script` command
-- and contains an MD5 version of your OMERO root users's password.
-- You should think about deleting it as soon as possible.
--
-- To create your database:
--
--     createdb omero
--     psql omero < %s
--

BEGIN;
                """ % (time.ctime(time.time()), sql_directory, script))
                self._copy(old_div(sql_directory,"schema.sql"), output, str)
                self._copy(
                    old_div(sql_directory,"data.sql"), output,
                    self._make_replace(password_hash, db_vers, db_patch))
                self._copy(old_div(sql_directory,"views.sql"), output, str)
                output.write("COMMIT;\n")

        finally:
            output.flush()
            if output != sys.stdout:
                output.close()

    @windows_warning
    def password(self, args):
        root_pass = None
        user_id = 0
        old_prompt = True
        if self._has_user_id(args):
            user_id = args.user_id
            if user_id != '0':  # For non-root, use new_prompt
                old_prompt = False
        try:
            root_pass = args.password
        except Exception as e:
            self.ctx.dbg("While getting arguments:" + str(e))
        if args.empty:
            password_hash = ""
        else:
            password_hash = self._get_password_hash(args, root_pass,
                                                    old_prompt)
        self.ctx.out("UPDATE password SET hash = '%s' "
                     "WHERE experimenter_id = %s;""" %
                     (password_hash, user_id))

    @windows_warning
    def loaddefaults(self):
        try:
            data2 = self.ctx.initData({})
            output = self.ctx.readDefaults()
            self.ctx.parsePropertyFile(data2, output)
        except Exception as e:
            self.ctx.dbg(str(e))
            data2 = None
        return data2

    @windows_warning
    def script(self, args):
        if args.posversion is not None:
            self.ctx.err("WARNING: Positional arguments are deprecated")
        if platform.system() == 'Windows':
            self.ctx.out("\n%s\n" % WINDOWS_WARNING)

        defaults = self.loaddefaults()
        db_vers = self._lookup("version", defaults, args)
        db_patch = self._lookup("patch", defaults, args)
        root_pass = args.password
        if root_pass:
            if args.pospassword:
                self.ctx.die(
                    1, "ERROR: Flag and positional argument given for password"
                    )
        else:
            root_pass = args.pospassword
        if root_pass:
            self.ctx.err("Using password from commandline")

        args.user_id = "0"
        sql = self._sql_directory(db_vers, db_patch)
        pwhash = self._get_password_hash(args, root_pass, True)
        self._create(sql, db_vers, db_patch, pwhash, args)

try:
    register("db", DatabaseControl, HELP)
except NameError:
    if __name__ == "__main__":
        cli = CLI()
        cli.register("db", DatabaseControl, HELP)
        cli.invoke(sys.argv[1:])



# -*- coding: utf-8 -*-
# **********************************************************************
#
# Copyright (c) 2003-2017 ZeroC, Inc. All rights reserved.
#
# This copy of Ice is licensed to you under the terms described in the
# ICE_LICENSE file included in this distribution.
#
# **********************************************************************
#
# Ice version 3.6.4
#
# <auto-generated>
#
# Generated from file `QuantumDef.ice'
#
# Warning: do not edit this file.
#
# </auto-generated>
#

from sys import version_info as _version_info_
import Ice, IcePy
import omero_model_IObject_ice
import omero_RTypes_ice
import omero_model_RTypes_ice
import omero_System_ice
import omero_Collections_ice

# Included module omero
_M_omero = Ice.openModule('omero')

# Included module omero.model
_M_omero.model = Ice.openModule('omero.model')

# Included module Ice
_M_Ice = Ice.openModule('Ice')

# Included module omero.sys
_M_omero.sys = Ice.openModule('omero.sys')

# Included module omero.api
_M_omero.api = Ice.openModule('omero.api')

# Start of module omero
__name__ = 'omero'

# Start of module omero.model
__name__ = 'omero.model'

if 'Details' not in _M_omero.model.__dict__:
    _M_omero.model._t_Details = IcePy.declareClass('::omero::model::Details')
    _M_omero.model._t_DetailsPrx = IcePy.declareProxy('::omero::model::Details')

if 'QuantumDef' not in _M_omero.model.__dict__:
    _M_omero.model.QuantumDef = Ice.createTempClass()
    class QuantumDef(_M_omero.model.IObject):
        def __init__(self, _id=None, _details=None, _loaded=False, _version=None, _cdStart=None, _cdEnd=None, _bitResolution=None):
            if Ice.getType(self) == _M_omero.model.QuantumDef:
                raise RuntimeError('omero.model.QuantumDef is an abstract class')
            _M_omero.model.IObject.__init__(self, _id, _details, _loaded)
            self._version = _version
            self._cdStart = _cdStart
            self._cdEnd = _cdEnd
            self._bitResolution = _bitResolution

        def ice_ids(self, current=None):
            return ('::Ice::Object', '::omero::model::IObject', '::omero::model::QuantumDef')

        def ice_id(self, current=None):
            return '::omero::model::QuantumDef'

        def ice_staticId():
            return '::omero::model::QuantumDef'
        ice_staticId = staticmethod(ice_staticId)

        def getVersion(self, current=None):
            pass

        def setVersion(self, theVersion, current=None):
            pass

        def getCdStart(self, current=None):
            pass

        def setCdStart(self, theCdStart, current=None):
            pass

        def getCdEnd(self, current=None):
            pass

        def setCdEnd(self, theCdEnd, current=None):
            pass

        def getBitResolution(self, current=None):
            pass

        def setBitResolution(self, theBitResolution, current=None):
            pass

        def __str__(self):
            return IcePy.stringify(self, _M_omero.model._t_QuantumDef)

        __repr__ = __str__

    _M_omero.model.QuantumDefPrx = Ice.createTempClass()
    class QuantumDefPrx(_M_omero.model.IObjectPrx):

        def getVersion(self, _ctx=None):
            return _M_omero.model.QuantumDef._op_getVersion.invoke(self, ((), _ctx))

        def begin_getVersion(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.QuantumDef._op_getVersion.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getVersion(self, _r):
            return _M_omero.model.QuantumDef._op_getVersion.end(self, _r)

        def setVersion(self, theVersion, _ctx=None):
            return _M_omero.model.QuantumDef._op_setVersion.invoke(self, ((theVersion, ), _ctx))

        def begin_setVersion(self, theVersion, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.QuantumDef._op_setVersion.begin(self, ((theVersion, ), _response, _ex, _sent, _ctx))

        def end_setVersion(self, _r):
            return _M_omero.model.QuantumDef._op_setVersion.end(self, _r)

        def getCdStart(self, _ctx=None):
            return _M_omero.model.QuantumDef._op_getCdStart.invoke(self, ((), _ctx))

        def begin_getCdStart(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.QuantumDef._op_getCdStart.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getCdStart(self, _r):
            return _M_omero.model.QuantumDef._op_getCdStart.end(self, _r)

        def setCdStart(self, theCdStart, _ctx=None):
            return _M_omero.model.QuantumDef._op_setCdStart.invoke(self, ((theCdStart, ), _ctx))

        def begin_setCdStart(self, theCdStart, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.QuantumDef._op_setCdStart.begin(self, ((theCdStart, ), _response, _ex, _sent, _ctx))

        def end_setCdStart(self, _r):
            return _M_omero.model.QuantumDef._op_setCdStart.end(self, _r)

        def getCdEnd(self, _ctx=None):
            return _M_omero.model.QuantumDef._op_getCdEnd.invoke(self, ((), _ctx))

        def begin_getCdEnd(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.QuantumDef._op_getCdEnd.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getCdEnd(self, _r):
            return _M_omero.model.QuantumDef._op_getCdEnd.end(self, _r)

        def setCdEnd(self, theCdEnd, _ctx=None):
            return _M_omero.model.QuantumDef._op_setCdEnd.invoke(self, ((theCdEnd, ), _ctx))

        def begin_setCdEnd(self, theCdEnd, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.QuantumDef._op_setCdEnd.begin(self, ((theCdEnd, ), _response, _ex, _sent, _ctx))

        def end_setCdEnd(self, _r):
            return _M_omero.model.QuantumDef._op_setCdEnd.end(self, _r)

        def getBitResolution(self, _ctx=None):
            return _M_omero.model.QuantumDef._op_getBitResolution.invoke(self, ((), _ctx))

        def begin_getBitResolution(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.QuantumDef._op_getBitResolution.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getBitResolution(self, _r):
            return _M_omero.model.QuantumDef._op_getBitResolution.end(self, _r)

        def setBitResolution(self, theBitResolution, _ctx=None):
            return _M_omero.model.QuantumDef._op_setBitResolution.invoke(self, ((theBitResolution, ), _ctx))

        def begin_setBitResolution(self, theBitResolution, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.QuantumDef._op_setBitResolution.begin(self, ((theBitResolution, ), _response, _ex, _sent, _ctx))

        def end_setBitResolution(self, _r):
            return _M_omero.model.QuantumDef._op_setBitResolution.end(self, _r)

        def checkedCast(proxy, facetOrCtx=None, _ctx=None):
            return _M_omero.model.QuantumDefPrx.ice_checkedCast(proxy, '::omero::model::QuantumDef', facetOrCtx, _ctx)
        checkedCast = staticmethod(checkedCast)

        def uncheckedCast(proxy, facet=None):
            return _M_omero.model.QuantumDefPrx.ice_uncheckedCast(proxy, facet)
        uncheckedCast = staticmethod(uncheckedCast)

        def ice_staticId():
            return '::omero::model::QuantumDef'
        ice_staticId = staticmethod(ice_staticId)

    _M_omero.model._t_QuantumDefPrx = IcePy.defineProxy('::omero::model::QuantumDef', QuantumDefPrx)

    _M_omero.model._t_QuantumDef = IcePy.declareClass('::omero::model::QuantumDef')

    _M_omero.model._t_QuantumDef = IcePy.defineClass('::omero::model::QuantumDef', QuantumDef, -1, (), True, False, _M_omero.model._t_IObject, (), (
        ('_version', (), _M_omero._t_RInt, False, 0),
        ('_cdStart', (), _M_omero._t_RInt, False, 0),
        ('_cdEnd', (), _M_omero._t_RInt, False, 0),
        ('_bitResolution', (), _M_omero._t_RInt, False, 0)
    ))
    QuantumDef._ice_type = _M_omero.model._t_QuantumDef

    QuantumDef._op_getVersion = IcePy.Operation('getVersion', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero._t_RInt, False, 0), ())
    QuantumDef._op_setVersion = IcePy.Operation('setVersion', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero._t_RInt, False, 0),), (), None, ())
    QuantumDef._op_getCdStart = IcePy.Operation('getCdStart', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero._t_RInt, False, 0), ())
    QuantumDef._op_setCdStart = IcePy.Operation('setCdStart', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero._t_RInt, False, 0),), (), None, ())
    QuantumDef._op_getCdEnd = IcePy.Operation('getCdEnd', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero._t_RInt, False, 0), ())
    QuantumDef._op_setCdEnd = IcePy.Operation('setCdEnd', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero._t_RInt, False, 0),), (), None, ())
    QuantumDef._op_getBitResolution = IcePy.Operation('getBitResolution', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero._t_RInt, False, 0), ())
    QuantumDef._op_setBitResolution = IcePy.Operation('setBitResolution', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero._t_RInt, False, 0),), (), None, ())

    _M_omero.model.QuantumDef = QuantumDef
    del QuantumDef

    _M_omero.model.QuantumDefPrx = QuantumDefPrx
    del QuantumDefPrx

# End of module omero.model

__name__ = 'omero'

# End of module omero
